@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -68,8 +68,6 @@ class NMatrixInt;
  * <i>d0</i>|<i>d1</i>|...|<i>dn</i>.  Note that this representation is
  * unique.
  *
- * \testpart
- *
  * \todo \optlong Look at using sparse matrices for storage of SNF and
  * the like.
  */
@@ -319,6 +317,14 @@ class REGINA_API NAbelianGroup : public ShareableObject {
          * @return \c true if and only if the two groups are isomorphic.
          */
         bool operator == (const NAbelianGroup& other) const;
+        /**
+         * Determines whether this and the given abelian group are
+         * non-isomorphic.
+         *
+         * @param other the group with which this should be compared.
+         * @return \c true if and only if the two groups are non-isomorphic.
+         */
+        bool operator != (const NAbelianGroup& other) const;
 
         /**
          * Writes a chunk of XML containing this abelian group.
@@ -413,6 +419,10 @@ inline bool NAbelianGroup::operator == (const NAbelianGroup& other) const {
     return (rank == other.rank && invariantFactors == other.invariantFactors);
 }
 
+inline bool NAbelianGroup::operator != (const NAbelianGroup& other) const {
+    return (rank != other.rank || invariantFactors != other.invariantFactors);
+}
+
 } // namespace regina
 
 #endif
