@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -48,6 +48,7 @@
 
 namespace regina {
 
+class NMatrixInt;
 class NTriangulation;
 class NXMLAngleStructureReader;
 
@@ -79,8 +80,6 @@ class NXMLAngleStructureReader;
  * The final element of the vector is the scaling member as described
  * above.
  *
- * \testpart
- *
  * \ifacespython Not present.
  */
 class REGINA_API NAngleStructureVector : public NRay {
@@ -98,21 +97,37 @@ class REGINA_API NAngleStructureVector : public NRay {
          * @param cloneMe the vector to clone.
          */
         NAngleStructureVector(const NVector<NLargeInteger>& cloneMe);
+
+        /**
+         * Creates a new set of angle structure equations for the given
+         * triangulation.
+         *
+         * Each equation will be represented as a row of the matrix, and
+         * each column will represent a coordinate in the underlying
+         * coordinate system (which is described in the NAngleStructureVector
+         * class notes).
+         *
+         * The returned matrix will be newly allocated and its destruction
+         * will be the responsibility of the caller of this routine.
+         *
+         * @param tri the triangulation upon which these angle structure
+         * equations will be based.
+         * @return a newly allocated set of equations.
+         */
+        static NMatrixInt* makeAngleEquations(const NTriangulation* tri);
 };
 
 /**
  * Represents an angle structure on a triangulation.
  * Once the underlying triangulation changes, this angle structure
  * is no longer valid.
- *
- * \testpart
  */
 class REGINA_API NAngleStructure : public ShareableObject {
     private:
         NAngleStructureVector* vector;
             /**< Stores (indirectly) the individual angles in this angle
              *   structure. */
-        NTriangulation* triangulation;
+        const NTriangulation* triangulation;
             /**< The triangulation on which this angle structure is placed. */
 
         mutable unsigned long flags;
@@ -143,7 +158,7 @@ class REGINA_API NAngleStructure : public ShareableObject {
          * @param newVector a vector containing the individual angles in the
          * angle structure.
          */
-        NAngleStructure(NTriangulation* triang,
+        NAngleStructure(const NTriangulation* triang,
             NAngleStructureVector* newVector);
         /**
          * Destroys this angle structure.
@@ -169,8 +184,8 @@ class REGINA_API NAngleStructure : public ShareableObject {
          * inclusive.
          * @param edgePair the number of the vertex splitting
          * representing the pair of edges holding the requested angle;
-         * this should be between 0 and 2 inclusive.  See ::vertexSplit
-         * and ::vertexSplitDefn for details regarding vertex splittings.
+         * this should be between 0 and 2 inclusive.  See regina::vertexSplit
+         * and regina::vertexSplitDefn for details regarding vertex splittings.
          * @return the requested angle scaled down by <i>pi</i>.
          */
         NRational getAngle(unsigned long tetIndex, int edgePair) const;
@@ -180,7 +195,7 @@ class REGINA_API NAngleStructure : public ShareableObject {
          *
          * @return the underlying triangulation.
          */
-        NTriangulation* getTriangulation() const;
+        const NTriangulation* getTriangulation() const;
 
         /**
          * Determines whether this is a strict angle structure.
@@ -238,6 +253,20 @@ class REGINA_API NAngleStructure : public ShareableObject {
          */
         bool isVeering() const;
 
+        /**
+         * Gives read-only access to the raw vector that sits beneath this
+         * angle structure.
+         *
+         * Generally users should not need this function.  However, it is
+         * provided here in case the need should arise (e.g., for reasons
+         * of efficiency).
+         *
+         * \ifacespython Not present.
+         *
+         * @return the underlying raw vector.
+         */
+        const NAngleStructureVector* rawVector() const;
+
         void writeTextShort(std::ostream& out) const;
 
         /**
@@ -275,7 +304,7 @@ inline NAngleStructureVector::NAngleStructureVector(
 
 // Inline functions for NAngleStructure
 
-inline NAngleStructure::NAngleStructure(NTriangulation* triang,
+inline NAngleStructure::NAngleStructure(const NTriangulation* triang,
         NAngleStructureVector* newVector) : vector(newVector),
         triangulation(triang), flags(0) {
 }
@@ -284,7 +313,7 @@ inline NAngleStructure::~NAngleStructure() {
     delete vector;
 }
 
-inline NTriangulation* NAngleStructure::getTriangulation() const {
+inline const NTriangulation* NAngleStructure::getTriangulation() const {
     return triangulation;
 }
 
@@ -306,6 +335,10 @@ inline bool NAngleStructure::isVeering() const {
     return ((flags & flagVeering) != 0);
 }
 
+inline const NAngleStructureVector* NAngleStructure::rawVector() const {
+    return vector;
+}
+
 } // namespace regina
 
 #endif
