@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -41,7 +41,6 @@
 #define __DIM2TRIANGULATION_H
 #endif
 
-#include <list>
 #include <memory>
 #include <vector>
 #include "regina-core.h"
@@ -667,125 +666,12 @@ class REGINA_API Dim2Triangulation : public NPacket,
          */
         /*@{*/
 
-        /**
-         * Determines if this triangulation is combinatorially
-         * isomorphic to the given triangulation.
-         *
-         * Specifically, this routine determines if there is a
-         * one-to-one and onto boundary complete combinatorial
-         * isomorphism from this triangulation to \a other.  Boundary
-         * complete isomorphisms are described in detail in the
-         * Dim2Isomorphism class notes.
-         *
-         * In particular, note that this triangulation and \a other must
-         * contain the same number of triangles for such an isomorphism
-         * to exist.
-         *
-         * If a boundary complete isomorphism is found, the details of
-         * this isomorphism are returned.  The isomorphism is newly
-         * constructed, and so to assist with memory management is
-         * returned as a std::auto_ptr.  Thus, to test whether an
-         * isomorphism exists without having to explicitly deal with the
-         * isomorphism itself, you can call
-         * <tt>if (isIsomorphicTo(other).get())</tt> and the newly
-         * created isomorphism (if it exists) will be automatically
-         * destroyed.
-         *
-         * @param other the triangulation to compare with this one.
-         * @return details of the isomorphism if the two triangulations
-         * are combinatorially isomorphic, or a null pointer otherwise.
-         */
-        std::auto_ptr<Dim2Isomorphism> isIsomorphicTo(
-            const Dim2Triangulation& other) const;
-
-        /**
-         * Determines if an isomorphic copy of this triangulation is
-         * contained within the given triangulation, possibly as a
-         * subcomplex of some larger component (or components).
-         *
-         * Specifically, this routine determines if there is a boundary
-         * incomplete combinatorial isomorphism from this triangulation
-         * to \a other.  Boundary incomplete isomorphisms are described
-         * in detail in the Dim2Isomorphism class notes.
-         *
-         * In particular, note that boundary edges of this triangulation
-         * need not correspond to boundary edges of \a other, and that
-         * \a other can contain more triangles than this triangulation.
-         *
-         * If a boundary incomplete isomorphism is found, the details of
-         * this isomorphism are returned.  The isomorphism is newly
-         * constructed, and so to assist with memory management is
-         * returned as a std::auto_ptr.  Thus, to test whether an
-         * isomorphism exists without having to explicitly deal with the
-         * isomorphism itself, you can call
-         * <tt>if (isContainedIn(other).get())</tt> and the newly
-         * created isomorphism (if it exists) will be automatically
-         * destroyed.
-         *
-         * If more than one such isomorphism exists, only one will be
-         * returned.  For a routine that returns all such isomorphisms,
-         * see findAllSubcomplexesIn().
-         *
-         * @param other the triangulation in which to search for an
-         * isomorphic copy of this triangulation.
-         * @return details of the isomorphism if such a copy is found,
-         * or a null pointer otherwise.
-         */
-        std::auto_ptr<Dim2Isomorphism> isContainedIn(
-            const Dim2Triangulation& other) const;
-
-        /**
-         * Finds all ways in which an isomorphic copy of this triangulation
-         * is contained within the given triangulation, possibly as a
-         * subcomplex of some larger component (or components).
-         *
-         * This routine behaves identically to isContainedIn(), except
-         * that instead of returning just one isomorphism (which may be
-         * boundary incomplete and need not be onto), all such isomorphisms
-         * are returned.
-         *
-         * See the isContainedIn() notes for additional information.
-         *
-         * The isomorphisms that are found will be inserted into the
-         * given list.  These isomorphisms will be newly created, and
-         * the caller of this routine is responsible for destroying
-         * them.  The given list will not be emptied before the new
-         * isomorphisms are inserted.
-         *
-         * \ifacespython Not present.
-         *
-         * @param other the triangulation in which to search for
-         * isomorphic copies of this triangulation.
-         * @param results the list in which any isomorphisms found will
-         * be stored.
-         * @return the number of isomorphisms that were found.
-         */
-        unsigned long findAllSubcomplexesIn(const Dim2Triangulation& other,
-                std::list<Dim2Isomorphism*>& results) const;
-
-        /**
-         * Relabel the triangles and their vertices so that this
-         * triangulation is in canonical form.  This is essentially
-         * the lexicographically smallest labelling when the edge
-         * gluings are written out in order.
-         *
-         * Two triangulations are isomorphic if and only if their canonical
-         * forms are identical.
-         *
-         * The lexicographic ordering assumes that the edge gluings are
-         * written in order of triangle index and then edge number.
-         * Each gluing is written as the destination triangle index
-         * followed by the gluing permutation (which in turn is written
-         * as the images of 0,1,2 in order).
-         *
-         * \pre This routine currently works only when the triangulation
-         * is connected.  It may be extended to work with disconnected
-         * triangulations in later versions of Regina.
-         *
-         * @return \c true if the triangulation was changed, or \c false
-         * if the triangulation was in canonical form to begin with.
-         */
-        bool makeCanonical();
+        using NGenericTriangulation<2>::isIdenticalTo;
+        using NGenericTriangulation<2>::isIsomorphicTo;
+        using NGenericTriangulation<2>::isContainedIn;
+        using NGenericTriangulation<2>::findAllIsomorphisms;
+        using NGenericTriangulation<2>::findAllSubcomplexesIn;
+        using NGenericTriangulation<2>::makeCanonical;
 
         /*@}*/
         /**
@@ -793,6 +679,8 @@ class REGINA_API Dim2Triangulation : public NPacket,
          */
         /*@{*/
 
+        using NGenericTriangulation<2>::isEmpty;
+
         /**
          * Always returns \c true.
          *
@@ -821,6 +709,23 @@ class REGINA_API Dim2Triangulation : public NPacket,
          * @return \c true if and only if this triangulation is closed.
          */
         bool isClosed() const;
+        /**
+         * Determines if this triangulation has any boundary edges.
+         *
+         * This routine is redundant in dimension two, since it returns
+         * \c true if and only if isClosed() returns \c false.
+         * It is provided simply for compatibility with higher-dimensional
+         * triangulation classes.
+         *
+         * @return \c true if and only if there are boundary edges.
+         */
+        bool hasBoundaryEdges() const;
+        /**
+         * Returns the number of boundary edges in this triangulation.
+         *
+         * @return the total number of boundary edges.
+         */
+        unsigned long getNumberOfBoundaryEdges() const;
         /**
          * Determines if this triangulation is orientable.
          *
@@ -874,6 +779,43 @@ class REGINA_API Dim2Triangulation : public NPacket,
          */
         bool isMinimal() const;
 
+        /*@}*/
+        /**
+         * \name Skeletal Transformations
+         */
+        /*@{*/
+
+        /**
+         * Checks the eligibility of and/or performs a 1-3 move
+         * upon the given triangle.
+         * This involves replacing one triangle with three triangles:
+         * each new triangle runs from one edge of
+         * the original triangle to a new common internal degree three vertex.
+         *
+         * This move can always be performed.  The \a check argument is
+         * present (as for other moves), but is simply ignored (since
+         * the move is always legal).  The \a perform argument is also
+         * present for consistency with other moves, but if it is set to
+         * \c false then this routine does nothing and returns no useful
+         * information.
+         *
+         * Note that after performing this move, all skeletal objects
+         * (edges, components, etc.) will be reconstructed, which means
+         * any pointers to old skeletal objects (such as the argument \a t)
+         * can no longer be used.
+         *
+         * \pre The given triangle is a triangle of this triangulation.
+         *
+         * @param t the triangle about which to perform the move.
+         * @param check this argument is ignored, since this move is
+         * always legal (see the notes above).
+         * @param perform \c true if we are to perform the move
+         * (defaults to \c true).
+         * @return \c true always.
+         */
+        bool oneThreeMove(Dim2Triangle* t, bool check = true,
+            bool perform = true);
+
         /*@}*/
         /**
          * \name Building Triangulations
@@ -953,58 +895,8 @@ class REGINA_API Dim2Triangulation : public NPacket,
          */
         /*@{*/
 
-        /**
-         * Constructs the isomorphism signature for this triangulation.
-         *
-         * An <i>isomorphism signature</i> is a compact text representation of
-         * a triangulation.  Unlike dehydrations for 3-manifold triangulations,
-         * an isomorphism signature uniquely determines a triangulation up
-         * to combinatorial isomorphism.  That is, two 2-manifold
-         * triangulations are combinatorially isomorphic if and only if
-         * their isomorphism signatures are the same.
-         *
-         * The isomorphism signature is constructed entirely of
-         * printable characters, and has length proportional to
-         * <tt>n log n</tt>, where \a n is the number of triangles.
-         *
-         * Isomorphism signatures are more general than dehydrations:
-         * they can be used with any triangulation (including closed,
-         * bounded and/or disconnected triangulations, as well
-         * as triangulations with large numbers of triangles).
-         *
-         * The time required to construct the isomorphism signature of a
-         * triangulation is <tt>O(n^2 log^2 n)</tt>.
-         *
-         * The routine fromIsoSig() can be used to recover a
-         * triangulation from an isomorphism signature.  The triangulation
-         * recovered might not be identical to the original, but it will be
-         * combinatorially isomorphic.
-         *
-         * If \a relabelling is non-null (i.e., it points to some
-         * Dim2Isomorphism pointer \a p), then it will be modified to point
-         * to a new Dim2Isomorphism that describes the precise relationship
-         * between this triangulation and the reconstruction from fromIsoSig().
-         * Specifically, the triangulation that is reconstructed from
-         * fromIsoSig() will be combinatorially identical to
-         * <tt>relabelling.apply(this)</tt>.
-         *
-         * \ifacespython The isomorphism argument is not present.
-         * Instead there are two routines: fromIsoSig(), which returns a
-         * string only, and fromIsoSigDetail(), which returns a pair
-         * (signature, relabelling).
-         *
-         * \pre If \a relabelling is non-null, then this triangulation
-         * must be non-empty and connected.  The facility to return a
-         * relabelling for disconnected triangulations may be added to
-         * Regina in a later release.
-         *
-         * @param relabelling if non-null, this will be modified to point to a
-         * new isomorphism describing the relationship between this
-         * triangulation and that reconstructed from fromIsoSig(), as
-         * described above.
-         * @return the isomorphism signature of this triangulation.
-         */
-        std::string isoSig(Dim2Isomorphism** relabelling = 0) const;
+        using NGenericTriangulation<2>::isoSig;
+
         /**
          * Returns C++ code that can be used with insertConstruction()
          * to reconstruct this triangulation.
@@ -1033,26 +925,7 @@ class REGINA_API Dim2Triangulation : public NPacket,
          */
         /*@{*/
 
-        /**
-         * Recovers a full triangulation from an isomorphism signature.
-         * See isoSig() for more information on isomorphism signatures.
-         *
-         * The triangulation that is returned will be newly created.
-         *
-         * Calling isoSig() followed by fromIsoSig() is not guaranteed to
-         * produce an identical triangulation to the original, but it
-         * \e is guaranteed to produce a combinatorially isomorphic
-         * triangulation.
-         *
-         * @param signature the isomorphism signature of the
-         * triangulation to construct.  Note that, unlike dehydration
-         * strings for 3-manifold triangulations, case is important for
-         * isomorphism signatures.
-         * @return a newly allocated triangulation if the reconstruction was
-         * successful, or null if the given string was not a valid
-         * isomorphism signature.
-         */
-        static Dim2Triangulation* fromIsoSig(const std::string& signature);
+        using NGenericTriangulation<2>::fromIsoSig;
         using NGenericTriangulation<2>::isoSigComponentSize;
 
         /*@}*/
@@ -1183,6 +1056,7 @@ class REGINA_API Dim2Triangulation : public NPacket,
         static bool compatibleTriangles(Dim2Triangle* src, Dim2Triangle* dest,
             NPerm3 p);
 
+    friend class regina::NGenericTriangulation<2>;
     friend class regina::Dim2Triangle;
     friend class regina::NXMLDim2TriangulationReader;
 };
@@ -1459,6 +1333,16 @@ inline bool Dim2Triangulation::isClosed() const {
     return boundaryComponents_.empty();
 }
 
+inline bool Dim2Triangulation::hasBoundaryEdges() const {
+    return ! isClosed();
+}
+
+inline unsigned long Dim2Triangulation::getNumberOfBoundaryEdges() const {
+    if (! calculatedSkeleton_)
+        calculateSkeleton();
+    return 2 * edges_.size() - 3 * triangles_.size();
+}
+
 inline bool Dim2Triangulation::isOrientable() const {
     if (! calculatedSkeleton_)
         calculateSkeleton();
@@ -1479,16 +1363,6 @@ inline NPacket* Dim2Triangulation::internalClonePacket(NPacket*) const {
     return new Dim2Triangulation(*this);
 }
 
-inline std::string Dim2Triangulation::isoSig(Dim2Isomorphism** relabelling)
-        const {
-    return NGenericTriangulation<2>::isoSig(*this, relabelling);
-}
-
-inline Dim2Triangulation* Dim2Triangulation::fromIsoSig(
-        const std::string& signature) {
-    return NGenericTriangulation<2>::fromIsoSig(signature);
-}
-
 } // namespace regina
 
 #endif
