@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -32,77 +32,33 @@
 
 /* end stub */
 
-#include <cstdio>
-#include <sys/stat.h>
-
 #include "foreign/pdf.h"
 #include "packet/npdf.h"
+#include <cstdio>
 
 namespace regina {
 
 NPDF* readPDF(const char* filename) {
-    // Use FILE* so we can call fstat().
+    NPDF* ans = new NPDF(filename);
 
-    // Open the file.
-    FILE* in = fopen(filename, "rb");
-    if (! in)
+    if (ans->isNull()) {
+        delete ans;
         return 0;
-
-    // Get the file size.
-    struct stat s;
-    if (fstat(fileno(in), &s)) {
-        fclose(in);
-        return 0;
-    }
-    size_t size = s.st_size;
-
-    if (size == 0) {
-        fclose(in);
-        return new NPDF();
-    }
-
-    // Read the file contents.
-    char* data = new char[size];
-    if (fread(data, 1, size, in) != size) {
-        fclose(in);
-        delete[] data;
-        return 0;
-    }
-
-    // Is there more to the file that we weren't expecting?
-    char c;
-    if (fread(&c, 1, 1, in) > 0) {
-        fclose(in);
-        delete[] data;
-        return 0;
-    }
-
-    // All good!
-    fclose(in);
-    return new NPDF(data, size, NPDF::OWN_NEW);
+    } else
+        return ans;
 }
 
 bool writePDF(const char* filename, const NPDF& pdf) {
-    // Use FILE* for symmetry with readPDF().
-
-    // Open the file.
-    FILE* out = fopen(filename, "wb");
-    if (!out)
-        return false;
-
-    // Is there anything to write?
-    const char* data = pdf.data();
-    if (data) {
-        size_t size = pdf.size();
-        if (fwrite(data, 1, size, out) != size) {
-            fclose(out);
+    if (pdf.isNull()) {
+        // Preserve old behaviour for backward compatibility: create an
+        // empty file.
+        FILE* out = fopen(filename, "wb");
+        if (! out)
             return false;
-        }
-    }
-
-    // All done.
-    fclose(out);
-    return true;
+        fclose(out);
+        return true;
+    } else
+        return pdf.savePDF(filename);
 }
 
 } // namespace regina
