@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -153,67 +153,11 @@ NTriangulation* readSnapPea(std::istream& in) {
 }
 
 bool writeSnapPea(const char* filename, const NTriangulation& tri) {
-    std::ofstream out(filename);
-    if (!out)
-        return false;
-    else {
-        writeSnapPea(out, tri);
-        return true;
-    }
+    return tri.saveSnapPea(filename);
 }
 
 void writeSnapPea(std::ostream& out, const NTriangulation& tri) {
-    // Write header information.
-    out << "% Triangulation\n";
-    if (tri.getPacketLabel().empty())
-        out << "Regina_Triangulation\n";
-    else
-        out << stringToToken(tri.getPacketLabel()) << '\n';
-
-    // Write general details.
-    out << "not_attempted 0.0\n";
-    out << "unknown_orientability\n";
-    out << "CS_unknown\n";
-
-    // Write cusps.
-    out << "0 0\n";
-
-    // Write tetrahedra.
-    out << tri.getNumberOfTetrahedra() << '\n';
-
-    int i, j;
-    for (NTriangulation::TetrahedronIterator it = tri.getTetrahedra().begin();
-            it != tri.getTetrahedra().end(); it++) {
-        // Although our precondition states that there are no boundary
-        // triangles, we test for this anyway.  If somebody makes a mistake and
-        // calls this routine with a bounded triangulation, we don't want
-        // to wind up calling tetrahedronIndex(0) and crashing.
-        for (i = 0; i < 4; i++)
-            if ((*it)->adjacentTetrahedron(i))
-                out << "   " << tri.tetrahedronIndex(
-                    (*it)->adjacentTetrahedron(i)) << ' ';
-            else
-                out << "   -1 ";
-        out << '\n';
-        for (i = 0; i < 4; i++)
-            out << ' ' << (*it)->adjacentGluing(i).str();
-        out << '\n';
-
-        // Incident cusps.
-        for (i = 0; i < 4; i++)
-            out << "  -1 ";
-        out << '\n';
-
-        // Meridians and longitudes.
-        for (i = 0; i < 4; i++) {
-            for (j = 0; j < 16; j++)
-                out << "  0";
-            out << '\n';
-        }
-
-        // Tetrahedron shape.
-        out << "0.0 0.0\n";
-    }
+    tri.snapPea(out);
 }
 
 } // namespace regina
