@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -137,8 +137,6 @@ class NSFSpace;
  * replace this class with a more general class that (amongst other
  * things) removes this inconsistency.
  *
- * \testpart
- *
  * \todo \opt Speed up homology calculations involving orientable base
  * spaces by adding rank afterwards, instead of adding generators for
  * genus into the presentation matrix.
@@ -251,6 +249,7 @@ class REGINA_API NGraphTriple : public NManifold {
         bool operator < (const NGraphTriple& compare) const;
 
         NAbelianGroup* getHomologyH1() const;
+        bool isHyperbolic() const;
         std::ostream& writeName(std::ostream& out) const;
         std::ostream& writeTeXName(std::ostream& out) const;
 
@@ -312,6 +311,10 @@ inline const NMatrix2& NGraphTriple::matchingReln(unsigned which) const {
     return matchingReln_[which];
 }
 
+inline bool NGraphTriple::isHyperbolic() const {
+    return false;
+}
+
 } // namespace regina
 
 #endif
