@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -60,12 +60,10 @@ class NTriangulation;
  * 3-manifold that may be in use is not of interest.
  *
  * Subclasses corresponding to the different types of 3-manifold must
- * override at least the virtual functions writeName() and writeTeXName().
+ * (of course) override all pure virtual functions.
  * They do not need to override writeTextShort() or writeTextLong()
  * since these routines are properly implemented in the base class
  * NManifold.
- *
- * \testpart
  */
 class REGINA_API NManifold : public ShareableObject {
     public:
@@ -138,6 +136,14 @@ class REGINA_API NManifold : public ShareableObject {
          */
         virtual NAbelianGroup* getHomologyH1() const;
 
+        /**
+         * Returns whether or not this is a finite-volume hyperbolic manifold.
+         *
+         * @return \c true if this is a finite-volume hyperbolic
+         * manifold, or \c false if not.
+         */
+        virtual bool isHyperbolic() const = 0;
+
         /**
          * Determines in a fairly ad-hoc fashion whether this representation
          * of this 3-manifold is "smaller" than the given representation
