@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -74,8 +74,6 @@ class NLensSpace;
  * to work with the old NExceptionalFibre class should be looked at closely
  * before being adapted to the new NSFSFibre class (i.e., it may require
  * more than just substituting class names).
- *
- * \testpart
  */
 struct REGINA_API NSFSFibre {
     long alpha;
@@ -191,8 +189,6 @@ REGINA_API std::ostream& operator << (std::ostream& out, const NSFSFibre& f);
  * being adapted to the new NSFSpace class (i.e., it may require more
  * than just substituting class names).
  *
- * \testpart
- *
  * \todo \featurelong Implement recognition of more common names.
  * \todo \featurelong Implement triangulation construction and homology
  * calculation for more Seifert fibred spaces.
@@ -744,6 +740,7 @@ class REGINA_API NSFSpace : public NManifold {
 
         NTriangulation* construct() const;
         NAbelianGroup* getHomologyH1() const;
+        bool isHyperbolic() const;
         std::ostream& writeName(std::ostream& out) const;
         std::ostream& writeTeXName(std::ostream& out) const;
         std::ostream& writeStructure(std::ostream& out) const;
@@ -922,6 +919,10 @@ inline void NSFSpace::reflect() {
     b_ = -b_ - static_cast<long>(nFibres_);
 }
 
+inline bool NSFSpace::isHyperbolic() const {
+    return false;
+}
+
 inline std::ostream& NSFSpace::writeName(std::ostream& out) const {
     return writeCommonName(out, false);
 }
