@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -73,8 +73,6 @@ namespace regina {
  * All optional NManifold routines except for construct() are implemented
  * for this class.
  *
- * \testpart
- *
  * \todo \feature Implement the == operator for finding conjugate and
  * inverse matrices.
  */
@@ -138,6 +136,7 @@ class REGINA_API NTorusBundle : public NManifold {
         const NMatrix2& getMonodromy() const;
 
         NAbelianGroup* getHomologyH1() const;
+        bool isHyperbolic() const;
         std::ostream& writeName(std::ostream& out) const;
         std::ostream& writeTeXName(std::ostream& out) const;
 
@@ -278,6 +277,10 @@ inline void NTorusBundle::subtractRCUp() {
     monodromy[1][1] += monodromy[1][0];
 }
 
+inline bool NTorusBundle::isHyperbolic() const {
+    return false;
+}
+
 } // namespace regina
 
 #endif
