@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -68,8 +68,6 @@ namespace regina {
  * This class is faster and sleeker than related classes such as NPerm4 and
  * NPerm5.  On the other hand, this class does not offer quite as rich an
  * interface as the others.
- *
- * \testfull
  */
 class REGINA_API NPerm3 {
     public:
@@ -102,7 +100,14 @@ class REGINA_API NPerm3 {
          * Specifically, the inverse of permutation <tt>S3[i]</tt> is
          * the permutation <tt>S3[ invS3[i] ]</tt>.
          */
-        static const int invS3[6];
+        static const unsigned invS3[6];
+
+        /**
+         * A dimension-agnostic alias for NPerm3::invS3.  In general, for
+         * each \a K the class NPermK will define an alias \a invSn
+         * that references the list of all permutations NPermK::invSK.
+         */
+        static const unsigned* invSn;
 
         /**
          * Contains all possible permutations of three elements in
