@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -65,8 +65,6 @@ namespace regina {
  * The internal code is an unsigned integer.  The lowest three bits represent
  * the image of 0, the next lowest three bits represent the image of 1 and so
  * on.
- *
- * \testfull
  */
 class REGINA_API NPerm5 {
     public:
@@ -110,6 +108,13 @@ class REGINA_API NPerm5 {
          */
         static const unsigned invS5[120];
 
+        /**
+         * A dimension-agnostic alias for NPerm5::invS5.  In general, for
+         * each \a K the class NPermK will define an alias \a invSn
+         * that references the list of all permutations NPermK::invSK.
+         */
+        static const unsigned* invSn;
+
         /**
          * Contains all possible permutations of four elements.
          * In each permutation, 4 maps to 4.
