@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -32,6 +32,8 @@
 
 /* end stub */
 
+#include <cstdio>
+#include <sys/stat.h>
 #include "packet/npdf.h"
 #include "utilities/base64.h"
 #include "utilities/xmlutils.h"
@@ -40,6 +42,50 @@
 
 namespace regina {
 
+NPDF::NPDF(const char* filename) : data_(0), size_(0), alloc_(OWN_NEW) {
+    // Use FILE* so we can call fstat().
+
+    // Open the file.
+    FILE* in = fopen(filename, "rb");
+    if (! in)
+        return;
+
+    // Get the file size.
+    struct stat s;
+    if (fstat(fileno(in), &s)) {
+        fclose(in);
+        return;
+    }
+    size_t size = s.st_size;
+
+    if (size == 0) {
+        fclose(in);
+        return;
+    }
+
+    // Read the file contents.
+    char* data = new char[size];
+    if (fread(data, 1, size, in) != size) {
+        fclose(in);
+        delete[] data;
+        return;
+    }
+
+    // Is there more to the file that we weren't expecting?
+    char c;
+    if (fread(&c, 1, 1, in) > 0) {
+        fclose(in);
+        delete[] data;
+        return;
+    }
+
+    // All good!
+    fclose(in);
+
+    data_ = data;
+    size_ = size;
+}
+
 void NPDF::reset() {
     ChangeEventSpan span(this);
 
@@ -84,6 +130,28 @@ void NPDF::reset(char* data, size_t size, OwnershipPolicy alloc) {
     }
 }
 
+bool NPDF::savePDF(const char* filename) const {
+    if (! data_)
+        return false;
+
+    // Use FILE* for symmetry with the PDF load routine.
+
+    // Open the file.
+    FILE* out = fopen(filename, "wb");
+    if (!out)
+        return false;
+
+    // Is there anything to write?
+    if (fwrite(data_, 1, size_, out) != size_) {
+        fclose(out);
+        return false;
+    }
+
+    // All done.
+    fclose(out);
+    return true;
+}
+
 void NPDF::writeXMLPacketData(std::ostream& out) const {
     if (! data_) {
         // We have an empty PDF packet.
