@@ -38,6 +38,7 @@
  */
 
 #include "kernel.h"
+#include "kernel_namespace.h"
 
 
 typedef struct
@@ -421,3 +422,4 @@ static void compute_cusp_Euler_characteristics(
         cusp->euler_characteristic /= 2;
     }
 }
+#include "end_namespace.h"
