@@ -92,6 +92,7 @@
 #define _terse_triangulation_
 
 #include "kernel.h"
+#include "kernel_namespace.h"
 
 struct TerseTriangulation
 {
@@ -112,4 +113,6 @@ struct TerseTriangulation
 
 };
 
+#include "end_namespace.h"
+
 #endif
