@@ -56,6 +56,7 @@
 
 #include "kernel.h"
 #include <stdio.h>  /* for sprintf() in check_neighbors_and_gluings() */
+#include "kernel_namespace.h"
 
 static void check_neighbors_and_gluings(Triangulation *manifold);
 static int  count_the_edge_classes(Triangulation *manifold);
@@ -79,8 +80,8 @@ void data_to_triangulation(
                     k,
                     l,
                     m;
-    Boolean         all_peripheral_curves_are_zero,
-                    finite_vertices_are_present;
+    /*    Boolean         all_peripheral_curves_are_zero*/
+    Boolean finite_vertices_are_present;
 
     /*
      *  Initialize *manifold_ptr to NULL.
@@ -153,7 +154,7 @@ void data_to_triangulation(
      *  Set up the Tetrahedra.
      */
 
-    all_peripheral_curves_are_zero  = TRUE;
+    /*    all_peripheral_curves_are_zero  = TRUE; */
     finite_vertices_are_present     = FALSE;
 
     for (i = 0; i < manifold->num_tetrahedra; i++)
@@ -190,8 +191,10 @@ void data_to_triangulation(
                         {
                             tet_array[i]->curve[j][k][l][m] = data->tetrahedron_data[i].curve[j][k][l][m];
     
+			    /*
                             if (data->tetrahedron_data[i].curve[j][k][l][m] != 0)
                                 all_peripheral_curves_are_zero = FALSE;
+			    */
                         }
         }
     }
@@ -291,16 +294,18 @@ void data_to_triangulation(
      *
      * find_complete_hyperbolic_structure(manifold);
      * do_Dehn_filling(manifold);
-     */
-
-    /*
+     *
+     *
+     *
      *  If we provided the basis and the manifold is hyperbolic,
      *  replace it with a shortest basis.
+     *
+     * if (all_peripheral_curves_are_zero == TRUE
+     * && (   manifold->solution_type[complete] == geometric_solution
+     *    || manifold->solution_type[complete] == nongeometric_solution))
+     *   install_shortest_bases(manifold);
+     *
      */
-    if (all_peripheral_curves_are_zero == TRUE
-     && (   manifold->solution_type[complete] == geometric_solution
-         || manifold->solution_type[complete] == nongeometric_solution))
-        install_shortest_bases(manifold);
 
     /*
      *  If the Chern-Simons invariant is present, compute the fudge factor.
@@ -784,6 +789,11 @@ void copy_triangulation(
     my_free(new_tet);
     my_free(new_edge);
     my_free(new_cusp);
+
+    /*
+     * Copy the dilog function pointer.
+     */
+    destination->dilog = source->dilog;
 }
 
 
@@ -823,6 +833,7 @@ void initialize_triangulation(
     manifold->CS_value[penultimate]     = 0.0;
     manifold->CS_fudge[ultimate]        = 0.0;
     manifold->CS_fudge[penultimate]     = 0.0;
+    manifold->dilog                     = NULL;
 
     initialize_tetrahedron(&manifold->tet_list_begin);
     initialize_tetrahedron(&manifold->tet_list_end);
@@ -1091,3 +1102,4 @@ Permutation compose_permutations(
 
     return result;
 }
+#include "end_namespace.h"
