@@ -10,6 +10,8 @@
 #include <string.h>
 #include "unix_file_io.h"
 
+#include "kernel_namespace.h"
+
 #define READ_OLD_FILE_FORMAT 0
 
 #define FALSE   0
@@ -43,7 +45,7 @@
  */ 
 #define whitespace(x) (x=='\040'||x=='\f'||x=='\n'||x=='\r'||x=='\t'||x=='\v')
  
-static int read_head(char *headbuf, char *str, int words) {
+static int read_head(char *headbuf, const char *str, int words) {
     char *ptr = headbuf;
     int n;
     for (n=0; n < words; n++) {
@@ -64,7 +66,7 @@ static int read_head(char *headbuf, char *str, int words) {
  * from a string.
  */
 
-static TriangulationData    *ReadNewFileFormat(char *buffer);
+static TriangulationData    *ReadNewFileFormat(const char *buffer);
 static void                 WriteNewFileFormat(FILE *fp, TriangulationData *data);
 static char                 *StringNewFileFormat(TriangulationData *data);
 
@@ -74,16 +76,16 @@ extern FuncResult           read_old_manifold(FILE *fp, Triangulation **manifold
 
 /* Modified 2010/7/27 by NMD to allow for Classic Mac OS and Windows line endings, as well Unix ones */
 
-static Boolean  is_eol_char(char *buffer);
+static Boolean  is_eol_char(const char *buffer);
 
-Boolean is_eol_char(char *buffer){
+Boolean is_eol_char(const char *buffer){
   return (*buffer == '\n') ||  (*buffer == '\r');
 }
 
 /* Modified 04/24/09 by Marc Culler to allow extracting a triangulation from a C string. */
 
 Triangulation *read_triangulation_from_string(
-    char    *file_data)
+    const char    *file_data)
 {
     TriangulationData   *theTriangulationData;
     Triangulation       *manifold;
@@ -104,12 +106,11 @@ Triangulation *read_triangulation_from_string(
 }
 
 Triangulation *read_triangulation(
-    char    *file_name)
+    const char    *file_name)
 {
     FILE            *fp;
     Boolean         theNewFormat;
     Triangulation   *manifold;
-    char            firstline[100];
 
     /*
      *  If the file_name is nonempty, read the file.
@@ -137,7 +138,7 @@ Triangulation *read_triangulation(
     if (theNewFormat == TRUE)
     {
         TriangulationData   *theTriangulationData;
-	long filesize;
+	long filesize = 0;
 	char *buffer;
 	  
 	if ( fseek(fp, 0, SEEK_END) != 0    ||
@@ -145,7 +146,7 @@ Triangulation *read_triangulation(
 	     fseek(fp, 0, SEEK_SET) != 0     )
 	    uFatalError("read_triangulation", "unix file io");
 
-	buffer = malloc(filesize + 1);
+	buffer = (char *)malloc(filesize + 1);
 	if ( buffer == NULL)
 	    uFatalError("read_triangulation", "unix file io");
 	if ( fread(buffer, filesize, 1, fp) != 1 )
@@ -183,7 +184,7 @@ Triangulation *read_triangulation(
 
 
 static TriangulationData *ReadNewFileFormat(
-    char *buffer)
+    const char *buffer)
 {
     char                *ptr;
     char                theScratchString[SBSIZE];
@@ -196,7 +197,8 @@ static TriangulationData *ReadNewFileFormat(
                         k,
                         v,
                         f;
-    size_t size = 0;
+    size_t              size = 0;
+    double              temp, temp_m, temp_l, temp_r, temp_i;
 
 
     /*
@@ -223,7 +225,7 @@ static TriangulationData *ReadNewFileFormat(
    i = 0;
    ptr = NULL;
    while (!is_eol_char(buffer)){
-       if (i + 2 > size){
+     if (i + 2 > (int) size){
 	   size += 100;
 	   ptr = (char *) realloc(ptr, size);
 	   if (ptr == NULL){
@@ -265,7 +267,9 @@ static TriangulationData *ReadNewFileFormat(
      *  Read the volume.
      */
     buffer += read_head(HeadBuffer, buffer, 1);
-    sscanf(HeadBuffer, "%lf", &theTriangulationData->volume);
+    sscanf(HeadBuffer, "%lf", &temp);
+    theTriangulationData->volume = temp;
+
 
     /*
      *  Read the orientability.
@@ -293,8 +297,10 @@ static TriangulationData *ReadNewFileFormat(
     else
         uFatalError("ReadNewFileFormat 5", "unix file io");
     if (theTriangulationData->CS_value_is_known == TRUE) {
-      buffer += read_head(HeadBuffer, buffer, 1); 
-      sscanf(HeadBuffer, "%lf", &theTriangulationData->CS_value);
+      buffer += read_head(HeadBuffer, buffer, 1);
+      sscanf(HeadBuffer, "%lf", &temp);
+      theTriangulationData->CS_value = temp;
+
       }
     else
         theTriangulationData->CS_value = 0.0;
@@ -314,12 +320,14 @@ static TriangulationData *ReadNewFileFormat(
         uFatalError("ReadNewFileFormat 6", "unix file io");
     for (i = 0; i < theTotalNumCusps; i++)
     {
-        buffer += read_head(HeadBuffer, buffer, 3); 
+        buffer += read_head(HeadBuffer, buffer, 3);
         if (sscanf(HeadBuffer, "%s%lf%lf",
 		   theScratchString,
-		   &theTriangulationData->cusp_data[i].m,
-		   &theTriangulationData->cusp_data[i].l) != 3)
+		   &temp_m,
+		   &temp_l) != 3)
             uFatalError("ReadNewFileFormat 7", "unix file io");
+	theTriangulationData->cusp_data[i].m = temp_m;
+	theTriangulationData->cusp_data[i].l=temp_l;
         switch (theScratchString[0])
         {
             case 't':
@@ -408,17 +416,19 @@ static TriangulationData *ReadNewFileFormat(
          */
 	buffer += read_head(HeadBuffer, buffer, 2);
         sscanf(HeadBuffer, "%lf%lf",
-	       &theTriangulationData->tetrahedron_data[i].filled_shape.real,
-	       &theTriangulationData->tetrahedron_data[i].filled_shape.imag);
+	       &temp_r,
+	       &temp_i);
+	theTriangulationData->tetrahedron_data[i].filled_shape.real = temp_r;
+	theTriangulationData->tetrahedron_data[i].filled_shape.imag = temp_i;
     }
 
     return theTriangulationData;
 }
 
 
-void write_triangulation(
+Boolean write_triangulation(
     Triangulation   *manifold,
-    char            *file_name)
+    const char            *file_name)
 {
     TriangulationData   *theTriangulationData;
     FILE                *fp;
@@ -433,7 +443,7 @@ void write_triangulation(
         if (fp == NULL)
         {
             printf("couldn't open %s\n", file_name);
-            return;
+            return FALSE;
         }
     }
     else
@@ -445,6 +455,7 @@ void write_triangulation(
 
     if (fp != stdout)
         fclose(fp);
+    return TRUE;
 }
 
 
@@ -497,7 +508,7 @@ static void WriteNewFileFormat(
     }
 
     if (data->solution_type != not_attempted)
-        fprintf(fp, "  %.8f\n", data->volume);
+        fprintf(fp, "  %.8f\n", (double)data->volume);
     else
         fprintf(fp, "  %.1f\n", 0.0);
 
@@ -513,7 +524,7 @@ static void WriteNewFileFormat(
     }
 
     if (data->CS_value_is_known == TRUE)
-        fprintf(fp, "CS_known %.16f\n", data->CS_value);
+        fprintf(fp, "CS_known %.16f\n", (double)data->CS_value);
     else
         fprintf(fp, "CS_unknown\n");
 
@@ -521,8 +532,8 @@ static void WriteNewFileFormat(
     for (i = 0; i < data->num_or_cusps + data->num_nonor_cusps; i++)
         fprintf(fp, "    %s %16.12f %16.12f\n",
             (data->cusp_data[i].topology == torus_cusp) ? "torus" : "Klein",
-            data->cusp_data[i].m,
-            data->cusp_data[i].l);
+		(double)data->cusp_data[i].m,
+		(double)data->cusp_data[i].l);
     fprintf(fp, "\n");
 
     fprintf(fp, "%d\n", data->num_tetrahedra);
@@ -555,8 +566,8 @@ static void WriteNewFileFormat(
 
         if (data->solution_type != not_attempted)
             fprintf(fp, "%16.12f %16.12f\n\n",
-                data->tetrahedron_data[i].filled_shape.real,
-                data->tetrahedron_data[i].filled_shape.imag);
+		    (double)data->tetrahedron_data[i].filled_shape.real,
+		    (double)data->tetrahedron_data[i].filled_shape.imag);
         else
             fprintf(fp, "%3.1f %3.1f\n\n", 0.0, 0.0);
     }
@@ -591,7 +602,7 @@ static char *StringNewFileFormat(
     char *p;
 
     size = 100*(10 + data->num_or_cusps + data->num_nonor_cusps + 8*data->num_tetrahedra);
-    buffer = malloc(size);
+    buffer = (char *)malloc(size);
     if ( buffer == NULL)
       uFatalError("StringNewFileFormat", "unix file io");
     p = buffer;
@@ -635,7 +646,7 @@ static char *StringNewFileFormat(
     }
 
     if (data->solution_type != not_attempted)
-        p += sprintf(p, "  %.8f\n", data->volume);
+        p += sprintf(p, "  %.8f\n", (double)data->volume);
     else
         p += sprintf(p, "  %.1f\n", 0.0);
 
@@ -651,7 +662,7 @@ static char *StringNewFileFormat(
     }
 
     if (data->CS_value_is_known == TRUE)
-        p += sprintf(p, "CS_known %.16f\n", data->CS_value);
+        p += sprintf(p, "CS_known %.16f\n", (double)data->CS_value);
     else
         p += sprintf(p, "CS_unknown\n");
 
@@ -659,8 +670,8 @@ static char *StringNewFileFormat(
     for (i = 0; i < data->num_or_cusps + data->num_nonor_cusps; i++)
         p += sprintf(p, "    %s %16.12f %16.12f\n",
             (data->cusp_data[i].topology == torus_cusp) ? "torus" : "Klein",
-            data->cusp_data[i].m,
-            data->cusp_data[i].l);
+		     (double)data->cusp_data[i].m,
+		     (double)data->cusp_data[i].l);
     p += sprintf(p, "\n");
 
     p += sprintf(p, "%d\n", data->num_tetrahedra);
@@ -693,10 +704,12 @@ static char *StringNewFileFormat(
 
         if (data->solution_type != not_attempted)
             p += sprintf(p, "%16.12f %16.12f\n\n",
-                data->tetrahedron_data[i].filled_shape.real,
-                data->tetrahedron_data[i].filled_shape.imag);
+		 (double)data->tetrahedron_data[i].filled_shape.real,
+		 (double)data->tetrahedron_data[i].filled_shape.imag);
         else
             p += sprintf(p, "%3.1f %3.1f\n\n", 0.0, 0.0);
    }
    return buffer;
 }
+
+#include "end_namespace.h"
