@@ -1,4 +1,5 @@
 #include "../kernel/kernel.h"
+#include "../kernel/kernel_namespace.h"
 
 /*
  * This file was originally named "gluing_equations.c" when distributed with
@@ -13,7 +14,6 @@
  *
  * */
 
-
 /*
 
         Returns a matrix with rows of the form
@@ -153,3 +153,4 @@ void free_cusp_equation(int* equation){
   my_free(equation);
 }
   
+#include "../kernel/end_namespace.h"
