@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -38,7 +38,9 @@
 #include "snappea/nsnappeatriangulation.h"
 #include "snappea/kernel/SnapPea.h"
 
-bool regina::NSnapPeaTriangulation::kernelMessages = false;
+bool regina::NSnapPeaTriangulation::kernelMessages_ = false;
+
+namespace regina { namespace snappea {
 
 /**
  * Supply bare-bones UI messaging functions for the SnapPea kernel to use.
@@ -46,10 +48,6 @@ bool regina::NSnapPeaTriangulation::kernelMessages = false;
  * See snappea/kernel/SnapPea.h for details on what each function should do.
  */
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
 void uAcknowledge(const char *message) {
     if (regina::NSnapPeaTriangulation::kernelMessagesEnabled())
         std::cout << message << std::endl;
@@ -70,13 +68,13 @@ int uQuery(const char *message, const int num_responses,
 
 void uFatalError(const char *function, const char *file) {
     std::cerr << "FATAL ERROR: " << file << ", " << function << std::endl;
-    exit(1);
+    throw regina::SnapPeaFatalError(function, file);
 }
 
 void uAbortMemoryFull(void) {
     std::cerr << "FATAL ERROR: Available memory has been exhausted."
         << std::endl;
-    exit(1);
+    throw regina::SnapPeaMemoryFull();
 }
 
 
@@ -84,7 +82,7 @@ void uPrepareMemFullMessage() {
     // Do nothing for now.
 }
 
-void uLongComputationBegins(char *message, Boolean /* is_abortable */) {
+void uLongComputationBegins(const char *message, Boolean /* is_abortable */) {
     if (regina::NSnapPeaTriangulation::kernelMessagesEnabled())
         std::cout << message << std::endl;
 }
@@ -96,7 +94,4 @@ FuncResult uLongComputationContinues() {
 void uLongComputationEnds() {
 }
 
-#ifdef __cplusplus
-} // extern "C"
-#endif
-
+} } // namespaces
