@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -186,7 +186,7 @@ NDiscSetSurface::~NDiscSetSurface() {
 
 NDiscSpec* NDiscSetSurface::adjacentDisc(const NDiscSpec& disc,
         NPerm4 arc, NPerm4& adjArc) const {
-    NTetrahedron* tet = triangulation->getTetrahedron(disc.tetIndex);
+    const NTetrahedron* tet = triangulation->getTetrahedron(disc.tetIndex);
     int arcFace = arc[3];
     if (tet->adjacentTetrahedron(arcFace) == 0)
         return 0;
