@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -42,16 +42,6 @@
 #include "triangulation/ntriangulation.h"
 #include "utilities/xmlutils.h"
 
-// Property IDs:
-#define PROPID_EULERCHARACTERISTIC 1
-#define PROPID_REALBOUNDARY 5
-#define PROPID_COMPACT 6
-#define PROPID_ORIENTABILITY 7
-#define PROPID_TWOSIDEDNESS 8
-#define PROPID_CONNECTEDNESS 9
-#define PROPID_CANCRUSH 10
-#define PROPID_SURFACENAME 100
-
 namespace regina {
 
 const int vertexSplit[4][4] = {
@@ -143,7 +133,7 @@ NNormalSurface* NNormalSurface::doubleSurface() const {
     return ans;
 }
 
-NNormalSurface::NNormalSurface(NTriangulation* triang,
+NNormalSurface::NNormalSurface(const NTriangulation* triang,
         NNormalSurfaceVector* newVector) :
         vector(newVector),
         triangulation(triang) {
@@ -170,7 +160,8 @@ void NNormalSurface::writeTextShort(std::ostream& out) const {
     }
 }
 
-bool NNormalSurfaceVector::hasMultipleOctDiscs(NTriangulation* triang) const {
+bool NNormalSurfaceVector::hasMultipleOctDiscs(const NTriangulation* triang)
+        const {
     unsigned long nTets = triang->getNumberOfTetrahedra();
     int oct;
     NLargeInteger coord;
@@ -188,7 +179,7 @@ bool NNormalSurfaceVector::hasMultipleOctDiscs(NTriangulation* triang) const {
     return false;
 }
 
-bool NNormalSurfaceVector::isCompact(NTriangulation* triang) const {
+bool NNormalSurfaceVector::isCompact(const NTriangulation* triang) const {
     unsigned long nTets = triang->getNumberOfTetrahedra();
     unsigned long tet;
     int type;
@@ -208,7 +199,7 @@ bool NNormalSurfaceVector::isCompact(NTriangulation* triang) const {
     return true;
 }
 
-bool NNormalSurfaceVector::isSplitting(NTriangulation* triang) const {
+bool NNormalSurfaceVector::isSplitting(const NTriangulation* triang) const {
     unsigned long nTets = triang->getNumberOfTetrahedra();
     unsigned long tet;
     int type;
@@ -231,7 +222,8 @@ bool NNormalSurfaceVector::isSplitting(NTriangulation* triang) const {
     return true;
 }
 
-NLargeInteger NNormalSurfaceVector::isCentral(NTriangulation* triang) const {
+NLargeInteger NNormalSurfaceVector::isCentral(const NTriangulation* triang)
+        const {
     unsigned long nTets = triang->getNumberOfTetrahedra();
     unsigned long tet;
     int type;
@@ -345,7 +337,7 @@ void NNormalSurface::calculateOctPosition() const {
     return;
 }
 
-void NNormalSurface::calculateEulerCharacteristic() const {
+void NNormalSurface::calculateEulerChar() const {
     unsigned long index, tot;
     int type;
     NLargeInteger ans = NLargeInteger::zero;
@@ -384,7 +376,7 @@ void NNormalSurface::calculateRealBoundary() const {
 
     unsigned long index;
     unsigned long tot = triangulation->getNumberOfTetrahedra();
-    NTetrahedron* tet;
+    const NTetrahedron* tet;
     int type, face;
 
     for (index = 0; index < tot; index++) {
@@ -422,38 +414,34 @@ void NNormalSurface::calculateRealBoundary() const {
 }
 
 #ifndef EXCLUDE_SNAPPEA
-NMatrixInt* NNormalSurface::boundarySlopes() const {
-    NTriangulation *tri = getTriangulation();
+NMatrixInt* NNormalSurface::boundaryIntersections() const {
+    // Make sure this is really a SnapPea triangulation.
+    const NSnapPeaTriangulation* snapPea =
+        dynamic_cast<const NSnapPeaTriangulation*>(getTriangulation());
+    if (! snapPea)
+        return 0;
 
     // Check the preconditions.
-    if (! tri->isOriented())
+    if (! snapPea->isOriented())
         return 0;
     if (vector->allowsAlmostNormal())
         return 0;
-    for (NTriangulation::VertexIterator it = tri->getVertices().begin();
-            it != tri->getVertices().end(); ++it) {
+    for (NTriangulation::VertexIterator it = snapPea->getVertices().begin();
+            it != snapPea->getVertices().end(); ++it) {
         if (! (*it)->isIdeal())
             return 0;
         if (! (*it)->isLinkOrientable())
             return 0;
-        if ((*it)->getLinkEulerCharacteristic() != 0)
+        if ((*it)->getLinkEulerChar() != 0)
             return 0;
     }
 
-    NSnapPeaTriangulation snapPea(*tri, false);
-    NMatrixInt* equations = snapPea.slopeEquations();
+    NMatrixInt* equations = snapPea->slopeEquations();
     if (! equations)
         return 0;
 
-    // Check that snappea hasn't changed the triangulation.
-    // It shouldn't, but if it does then the matrix we obtain cannot be used.
-    if (! snapPea.verifyTriangulation(*tri)) {
-        delete equations;
-        return 0;
-    }
-
     unsigned long cusps = equations->rows() / 2;
-    unsigned long numTet = tri->getNumberOfTetrahedra();
+    unsigned long numTet = snapPea->getNumberOfTetrahedra();
     NMatrixInt* slopes = new NMatrixInt(cusps, 2);
     for(unsigned int i=0; i < cusps; i++) {
         NLargeInteger meridian; // constructor sets this to 0
@@ -514,12 +502,12 @@ void NNormalSurface::writeXMLData(std::ostream& out) const {
 // Default implementations for oriented surfaces. Returns zero as any
 // coordinate system which supports orientation should override these.
 NLargeInteger NNormalSurfaceVector::getOrientedTriangleCoord(
-        unsigned long, int, NTriangulation*, bool) const {
+        unsigned long, int, const NTriangulation*, bool) const {
     return NLargeInteger::zero;
 };
 
 NLargeInteger NNormalSurfaceVector::getOrientedQuadCoord(
-        unsigned long, int, NTriangulation*, bool) const {
+        unsigned long, int, const NTriangulation*, bool) const {
     return NLargeInteger::zero;
 };
 
