@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -49,7 +49,7 @@ namespace regina {
 unsigned long NTriangulation::splitIntoComponents(NPacket* componentParent,
         bool setLabels) {
     // Knock off the empty triangulation first.
-    if (tetrahedra.empty())
+    if (tetrahedra_.empty())
         return 0;
 
     if (! componentParent)
@@ -66,7 +66,7 @@ unsigned long NTriangulation::splitIntoComponents(NPacket* componentParent,
         newTris[whichComp] = new NTriangulation();
 
     // Clone the tetrahedra, sorting them into the new components.
-    unsigned long nTets = tetrahedra.size();
+    unsigned long nTets = tetrahedra_.size();
 
     NTetrahedron** newTets = new NTetrahedron*[nTets];
     NTetrahedron *tet, *adjTet;
@@ -76,12 +76,12 @@ unsigned long NTriangulation::splitIntoComponents(NPacket* componentParent,
 
     for (tetPos = 0; tetPos < nTets; tetPos++)
         newTets[tetPos] =
-            newTris[componentIndex(tetrahedra[tetPos]->getComponent())]->
-            newTetrahedron(tetrahedra[tetPos]->getDescription());
+            newTris[componentIndex(tetrahedra_[tetPos]->getComponent())]->
+            newTetrahedron(tetrahedra_[tetPos]->getDescription());
 
     // Clone the tetrahedron gluings also.
     for (tetPos = 0; tetPos < nTets; tetPos++) {
-        tet = tetrahedra[tetPos];
+        tet = tetrahedra_[tetPos];
         for (face = 0; face < 4; face++) {
             adjTet = tet->adjacentTetrahedron(face);
             if (adjTet) {
@@ -114,15 +114,17 @@ unsigned long NTriangulation::splitIntoComponents(NPacket* componentParent,
     return whichComp;
 }
 
-unsigned long NTriangulation::connectedSumDecomposition(NPacket* primeParent,
+long NTriangulation::connectedSumDecomposition(NPacket* primeParent,
         bool setLabels) {
     // Precondition checks.
-    if (! (isValid() && isClosed() && isOrientable() && isConnected()))
+    if (! (isValid() && isClosed() && isConnected()))
         return 0;
 
     if (! primeParent)
         primeParent = this;
 
+    bool initOrientable = isOrientable();
+
     // Make a working copy, simplify and record the initial homology.
     NTriangulation* working = new NTriangulation(*this);
     working->intelligentSimplify();
@@ -149,7 +151,7 @@ unsigned long NTriangulation::connectedSumDecomposition(NPacket* primeParent,
             toProcess.getFirstTreeChild()))) {
         // INV: Our triangulation is the connected sum of all the
         // children of toProcess, all the elements of primeComponents
-        // and possibly some copies of S2xS1, RP3 and/or L(3,1).
+        // and possibly some copies of S2xS1, S2x~S1, RP3, and/or L(3,1).
 
         // Work with the last child.
         processing->makeOrphan();
@@ -161,6 +163,23 @@ unsigned long NTriangulation::connectedSumDecomposition(NPacket* primeParent,
             delete sphere;
             delete processing;
 
+            if (! crushed->isValid()) {
+                // We must have had an embedded two-sided projective plane.
+                // Abort.
+
+                delete crushed;
+
+                std::list<NTriangulation*>::iterator it;
+                for (it = primeComponents.begin();
+                        it != primeComponents.end(); ++it)
+                    delete *it;
+
+                // All children of toProcess will be deleted automatically
+                // with toProcess itself (which is on the stack).
+
+                return -1;
+            }
+
             crushed->intelligentSimplify();
 
             // Insert each component of the crushed triangulation back
@@ -177,43 +196,49 @@ unsigned long NTriangulation::connectedSumDecomposition(NPacket* primeParent,
             // We have no non-trivial normal 2-spheres!
             // The triangulation is 0-efficient (and prime).
             // Is it a 3-sphere?
-            if (processing->getNumberOfVertices() > 1) {
-                // Proposition 5.1 of Jaco & Rubinstein's 0-efficiency
-                // paper:  If a closed orientable triangulation T is
-                // 0-efficient then either T has one vertex or T is a
-                // 3-sphere with precisely two vertices.
-                //
-                // It follows then that this is a 3-sphere.
-                // Toss it away.
-                delete sphere;
-                delete processing;
+            if (! processing->isOrientable()) {
+                // Definitely not a sphere.
+                primeComponents.push_back(processing);
             } else {
-                // Now we have a closed orientable one-vertex 0-efficient
-                // triangulation.
-                // We have to look for an almost normal sphere.
-                //
-                // From the proof of Proposition 5.12 in Jaco & Rubinstein's
-                // 0-efficiency paper, we see that we can restrict our
-                // search to octagonal almost normal surfaces.
-                // Furthermore, from the result in the quadrilateral-octagon
-                // coordinates paper, we can restrict this search further
-                // to vertex octagonal almost normal surfaces in
-                // quadrilateral-octagonal space.
-                sphere = processing->hasOctagonalAlmostNormalSphere();
-                if (sphere) {
-                    // It's a 3-sphere.  Toss this component away.
+                // Orientable, and so possibly a sphere.  Test this precisely.
+                if (processing->getNumberOfVertices() > 1) {
+                    // Proposition 5.1 of Jaco & Rubinstein's 0-efficiency
+                    // paper:  If a closed orientable triangulation T is
+                    // 0-efficient then either T has one vertex or T is a
+                    // 3-sphere with precisely two vertices.
+                    //
+                    // It follows then that this is a 3-sphere.
+                    // Toss it away.
                     delete sphere;
                     delete processing;
                 } else {
-                    // It's a non-trivial prime component!
-                    primeComponents.push_back(processing);
+                    // Now we have a closed orientable one-vertex 0-efficient
+                    // triangulation.
+                    // We have to look for an almost normal sphere.
+                    //
+                    // From the proof of Proposition 5.12 in Jaco & Rubinstein's
+                    // 0-efficiency paper, we see that we can restrict our
+                    // search to octagonal almost normal surfaces.
+                    // Furthermore, from the result in the quadrilateral-octagon
+                    // coordinates paper, we can restrict this search further
+                    // to vertex octagonal almost normal surfaces in
+                    // quadrilateral-octagonal space.
+                    sphere = processing->hasOctagonalAlmostNormalSphere();
+                    if (sphere) {
+                        // It's a 3-sphere.  Toss this component away.
+                        delete sphere;
+                        delete processing;
+                    } else {
+                        // It's a non-trivial prime component!
+                        primeComponents.push_back(processing);
+                    }
                 }
             }
         }
     }
 
-    // Run a final homology check and put back our missing S2xS1, RP3
-    // and L(3,1) terms.
+    // Run a final homology check and put back our missing S2xS1, S2x~S1,
+    // RP3 and L(3,1) terms.
     unsigned long finalZ = 0, finalZ2 = 0, finalZ3 = 0;
     for (std::list<NTriangulation*>::iterator it = primeComponents.begin();
             it != primeComponents.end(); it++) {
@@ -225,10 +250,21 @@ unsigned long NTriangulation::connectedSumDecomposition(NPacket* primeParent,
 
     while (finalZ++ < initZ) {
         working = new NTriangulation();
-        working->insertLayeredLensSpace(0, 1);
+        if (initOrientable) {
+            // Build S2 x S1.
+            working->insertLayeredLensSpace(0, 1);
+        } else {
+            // Build S2 x~ S1.
+            NTetrahedron* t0 = working->newTetrahedron();
+            NTetrahedron* t1 = working->newTetrahedron();
+            t0->joinTo(0, t1, NPerm4(0, 1, 3, 2));
+            t0->joinTo(1, t1, NPerm4(0, 1, 3, 2));
+            t0->joinTo(2, t1, NPerm4(1, 3, 2, 0));
+            t0->joinTo(3, t1, NPerm4(2, 0, 1, 3));
+        }
         primeComponents.push_back(working);
-        irreducible = false; // Implied by the S2xS1 summand.
-        zeroEfficient = false; // Implied by the S2xS1 summand.
+        irreducible_ = false; // Implied by the S2xS1 or S2x~S1 summand.
+        zeroEfficient_ = false; // Implied by the S2xS1 or S2x~S1 summand.
     }
     while (finalZ2++ < initZ2) {
         working = new NTriangulation();
@@ -260,35 +296,35 @@ unsigned long NTriangulation::connectedSumDecomposition(NPacket* primeParent,
 
     // Set irreducibility while we're at it.
     if (whichComp > 1) {
-        threeSphere = false;
-        irreducible = false;
-        zeroEfficient = false;
+        threeSphere_ = false;
+        irreducible_ = false;
+        zeroEfficient_ = false;
     } else if (whichComp == 1) {
-        threeSphere = false;
-        if (! irreducible.known()) {
-            // If our manifold is S2xS1 then it is *not* irreducible;
+        threeSphere_ = false;
+        if (! irreducible_.known()) {
+            // If our manifold is S2xS1 or S2x~S1 then it is *not* irreducible;
             // however, in this case we will have already set irreducible
-            // to false when putting back the S2xS1 summands above (and
-            // therefore irreducible.known() will be true).
-            irreducible = true;
+            // to false when putting back the S2xS1 or S2x~S1 summands above
+            // (and therefore irreducible.known() will be true).
+            irreducible_ = true;
         }
     } else if (whichComp == 0) {
-        threeSphere = true;
-        irreducible = true;
-        haken = false;
+        threeSphere_ = true;
+        irreducible_ = true;
+        haken_ = false;
     }
 
     return whichComp;
 }
 
 bool NTriangulation::isThreeSphere() const {
-    if (threeSphere.known())
-        return threeSphere.value();
+    if (threeSphere_.known())
+        return threeSphere_.value();
 
     // Basic property checks.
     if (! (isValid() && isClosed() && isOrientable() && isConnected() &&
             getNumberOfTetrahedra() > 0)) {
-        threeSphere = false;
+        threeSphere_ = false;
         return false;
     }
 
@@ -299,11 +335,12 @@ bool NTriangulation::isThreeSphere() const {
 
     // The Poincare conjecture!
     if (working->getFundamentalGroup().getNumberOfGenerators() == 0) {
-        threeSphere = true;
+        threeSphere_ = true;
+        delete working;
 
         // Some other things that come for free:
-        irreducible = true;
-        haken = false;
+        irreducible_ = true;
+        haken_ = false;
 
         return true;
     }
@@ -311,7 +348,7 @@ bool NTriangulation::isThreeSphere() const {
     // We could still have a trivial group but not know it.
     // At least we can at least check homology precisely.
     if (! working->getHomologyH1().isTrivial()) {
-        threeSphere = false;
+        threeSphere_ = false;
         delete working;
         return false;
     }
@@ -385,7 +422,7 @@ bool NTriangulation::isThreeSphere() const {
                     delete processing;
                 } else {
                     // It's not a 3-sphere.  We're done!
-                    threeSphere = false;
+                    threeSphere_ = false;
                     delete processing;
                     return false;
                 }
@@ -394,22 +431,22 @@ bool NTriangulation::isThreeSphere() const {
     }
 
     // Our triangulation is the connected sum of 0 components!
-    threeSphere = true;
+    threeSphere_ = true;
 
     // Some other things that we get for free:
-    irreducible = true;
-    haken = false;
+    irreducible_ = true;
+    haken_ = false;
 
     return true;
 }
 
 bool NTriangulation::knowsThreeSphere() const {
-    if (threeSphere.known())
+    if (threeSphere_.known())
         return true;
 
     // Run some very fast prelimiary tests before we give up and say no.
     if (! (isValid() && isClosed() && isOrientable() && isConnected())) {
-        threeSphere = false;
+        threeSphere_ = false;
         return true;
     }
 
@@ -418,14 +455,14 @@ bool NTriangulation::knowsThreeSphere() const {
 }
 
 bool NTriangulation::isBall() const {
-    if (threeBall.known())
-        return threeBall.value();
+    if (threeBall_.known())
+        return threeBall_.value();
 
     // Basic property checks.
     if (! (isValid() && hasBoundaryTriangles() && isOrientable() && isConnected()
-            && boundaryComponents.size() == 1
-            && boundaryComponents.front()->getEulerCharacteristic() == 2)) {
-        threeBall = false;
+            && boundaryComponents_.size() == 1
+            && boundaryComponents_.front()->getEulerChar() == 2)) {
+        threeBall_ = false;
         return false;
     }
 
@@ -442,19 +479,19 @@ bool NTriangulation::isBall() const {
     // Simplify again in case our coning was inefficient.
     working.intelligentSimplify();
 
-    threeBall = working.isThreeSphere();
-    return threeBall.value();
+    threeBall_ = working.isThreeSphere();
+    return threeBall_.value();
 }
 
 bool NTriangulation::knowsBall() const {
-    if (threeBall.known())
+    if (threeBall_.known())
         return true;
 
     // Run some very fast prelimiary tests before we give up and say no.
     if (! (isValid() && hasBoundaryTriangles() && isOrientable() && isConnected()
-            && boundaryComponents.size() == 1
-            && boundaryComponents.front()->getEulerCharacteristic() == 2)) {
-        threeBall = false;
+            && boundaryComponents_.size() == 1
+            && boundaryComponents_.front()->getEulerChar() == 2)) {
+        threeBall_ = false;
         return true;
     }
 
@@ -463,15 +500,15 @@ bool NTriangulation::knowsBall() const {
 }
 
 bool NTriangulation::isSolidTorus() const {
-    if (solidTorus.known())
-        return solidTorus.value();
+    if (solidTorus_.known())
+        return solidTorus_.value();
 
     // Basic property checks.
     if (! (isValid() && isOrientable() && isConnected() &&
-            boundaryComponents.size() == 1 &&
-            boundaryComponents.front()->getEulerCharacteristic() == 0 &&
-            boundaryComponents.front()->isOrientable()))
-        return (solidTorus = false);
+            boundaryComponents_.size() == 1 &&
+            boundaryComponents_.front()->getEulerChar() == 0 &&
+            boundaryComponents_.front()->isOrientable()))
+        return (solidTorus_ = false);
 
     // If it's ideal, make it a triangulation with real boundary.
     // If it's not ideal, clone it anyway so we can modify it.
@@ -485,7 +522,7 @@ bool NTriangulation::isSolidTorus() const {
     // Check homology.
     if (! (working->getHomologyH1().isZ())) {
         delete working;
-        return (solidTorus = false);
+        return (solidTorus_ = false);
     }
 
     // So:
@@ -521,7 +558,7 @@ bool NTriangulation::isSolidTorus() const {
         if (! s) {
             // No non-trivial normal disc.  This cannot be a solid torus.
             delete working;
-            return (solidTorus = false);
+            return (solidTorus_ = false);
         }
 
         // Crush it and see what happens.
@@ -545,7 +582,7 @@ bool NTriangulation::isSolidTorus() const {
                 // Must be a 3-sphere, or else we didn't have a solid torus.
                 if (! comp->isThreeSphere()) {
                     delete crushed;
-                    return (solidTorus = false);
+                    return (solidTorus_ = false);
                 }
             } else if (comp->getNumberOfBoundaryComponents() > 1) {
                 // Multiple boundaries on the same component.
@@ -556,14 +593,13 @@ bool NTriangulation::isSolidTorus() const {
 
                 // At any rate, it means we did not have a solid torus.
                 delete crushed;
-                return (solidTorus = false);
-            } else if (comp->getBoundaryComponent(0)->
-                    getEulerCharacteristic() == 2) {
+                return (solidTorus_ = false);
+            } else if (comp->getBoundaryComponent(0)->getEulerChar() == 2) {
                 // A component with sphere boundary.
                 // Must be a 3-ball, or else we didn't have a solid torus.
                 if (! comp->isBall()) {
                     delete crushed;
-                    return (solidTorus = false);
+                    return (solidTorus_ = false);
                 }
             } else {
                 // The only other possibility is a component with torus
@@ -588,7 +624,7 @@ bool NTriangulation::isSolidTorus() const {
             // (and we crushed and/or cut along a compressing disc
             // during the crushing operation).
             delete crushed;
-            return (solidTorus = true);
+            return (solidTorus_ = true);
         }
 
         // We have the original manifold in working, but this time with
@@ -599,23 +635,23 @@ bool NTriangulation::isSolidTorus() const {
 }
 
 bool NTriangulation::knowsSolidTorus() const {
-    if (solidTorus.known())
+    if (solidTorus_.known())
         return true;
 
     // Run some very fast prelimiary tests before we give up and say no.
     if (! (isValid() && isOrientable() && isConnected())) {
-        solidTorus = false;
+        solidTorus_ = false;
         return true;
     }
 
-    if (boundaryComponents.size() != 1) {
-        solidTorus = false;
+    if (boundaryComponents_.size() != 1) {
+        solidTorus_ = false;
         return true;
     }
 
-    if (boundaryComponents.front()->getEulerCharacteristic() != 0 ||
-            (! boundaryComponents.front()->isOrientable())) {
-        solidTorus = false;
+    if (boundaryComponents_.front()->getEulerChar() != 0 ||
+            (! boundaryComponents_.front()->isOrientable())) {
+        solidTorus_ = false;
         return true;
     }
 
@@ -631,7 +667,7 @@ NPacket* NTriangulation::makeZeroEfficient() {
     else
         connSum->setPacketLabel(getPacketLabel() + " - Decomposition");
 
-    unsigned long ans = connectedSumDecomposition(connSum, true);
+    long ans = connectedSumDecomposition(connSum, true);
     if (ans > 1) {
         // Composite!
         return connSum;
@@ -657,8 +693,8 @@ NPacket* NTriangulation::makeZeroEfficient() {
 }
 
 bool NTriangulation::isIrreducible() const {
-    if (irreducible.known())
-        return irreducible.value();
+    if (irreducible_.known())
+        return irreducible_.value();
 
     // Precondition checks.
     if (! (isValid() && isClosed() && isOrientable() && isConnected()))
@@ -756,10 +792,10 @@ bool NTriangulation::isIrreducible() const {
                     // them later).
                     if (summands > 0) {
                         // We have found more than one prime component.
-                        threeSphere = false; // Implied by reducibility.
-                        zeroEfficient = false; // Implied by reducibility.
+                        threeSphere_ = false; // Implied by reducibility.
+                        zeroEfficient_ = false; // Implied by reducibility.
                         delete processing;
-                        return (irreducible = false);
+                        return (irreducible_ = false);
                     }
                     ++summands;
 
@@ -782,44 +818,44 @@ bool NTriangulation::isIrreducible() const {
     if (Z > 0) {
         // There were S2xS1 summands that were crushed away.
         // The manifold must be reducible.
-        threeSphere = false; // Implied by reducibility.
-        zeroEfficient = false; // Implied by reducibility.
-        return (irreducible = false);
+        threeSphere_ = false; // Implied by reducibility.
+        zeroEfficient_ = false; // Implied by reducibility.
+        return (irreducible_ = false);
     }
     if (summands + Z2 + Z3 > 1) {
         // At least two summands were found and/or crushed away: the
         // manifold must be composite.
-        threeSphere = false; // Implied by reducibility.
-        zeroEfficient = false; // Implied by reducibility.
-        return (irreducible = false);
+        threeSphere_ = false; // Implied by reducibility.
+        zeroEfficient_ = false; // Implied by reducibility.
+        return (irreducible_ = false);
     }
 
     // There are no S2xS1 summands, and the manifold is prime.
-    return (irreducible = true);
+    return (irreducible_ = true);
 }
 
 bool NTriangulation::knowsIrreducible() const {
-    return irreducible.known();
+    return irreducible_.known();
 }
 
 bool NTriangulation::hasCompressingDisc() const {
-    if (compressingDisc.known())
-        return compressingDisc.value();
+    if (compressingDisc_.known())
+        return compressingDisc_.value();
 
     // Some sanity checks; also enforce preconditions.
     if (! hasBoundaryTriangles())
-        return (compressingDisc = false);
+        return (compressingDisc_ = false);
     if ((! isValid()) || isIdeal())
-        return (compressingDisc = false);
+        return (compressingDisc_ = false);
 
     long minBdryEuler = 2;
-    for (BoundaryComponentIterator it = boundaryComponents.begin();
-            it != boundaryComponents.end(); ++it) {
-        if ((*it)->getEulerCharacteristic() < minBdryEuler)
-            minBdryEuler = (*it)->getEulerCharacteristic();
+    for (BoundaryComponentIterator it = boundaryComponents_.begin();
+            it != boundaryComponents_.end(); ++it) {
+        if ((*it)->getEulerChar() < minBdryEuler)
+            minBdryEuler = (*it)->getEulerChar();
     }
     if (minBdryEuler == 2)
-        return (compressingDisc = false);
+        return (compressingDisc_ = false);
 
     // Off we go.
     // Work with a simplified triangulation.
@@ -829,7 +865,7 @@ bool NTriangulation::hasCompressingDisc() const {
     // Try for a fast answer first.
     if (use->hasSimpleCompressingDisc()) {
         delete use;
-        return (compressingDisc = true);
+        return (compressingDisc_ = true);
     }
 
     // Nope.  Decide whether we can use the fast linear programming
@@ -858,13 +894,13 @@ bool NTriangulation::hasCompressingDisc() const {
                     for (unsigned long i = 0; i < nSurfaces; ++i) {
                         if (q->getSurface(i)->isCompressingDisc(true)) {
                             delete use;
-                            return (compressingDisc = true);
+                            return (compressingDisc_ = true);
                         }
                     }
 
                     // No compressing discs!
                     delete use;
-                    return (compressingDisc = false);
+                    return (compressingDisc_ = false);
                 }
             }
 
@@ -872,7 +908,7 @@ bool NTriangulation::hasCompressingDisc() const {
             if (! search.find()) {
                 // No compressing discs!
                 delete use;
-                return (compressingDisc = false);
+                return (compressingDisc_ = false);
             }
 
             ans = search.buildSurface();
@@ -884,7 +920,7 @@ bool NTriangulation::hasCompressingDisc() const {
             comp = static_cast<NTriangulation*>(crush->getFirstTreeChild());
             while (comp) {
                 if (comp->getNumberOfBoundaryComponents() == 1 &&
-                        comp->getBoundaryComponent(0)->getEulerCharacteristic()
+                        comp->getBoundaryComponent(0)->getEulerChar()
                         == minBdryEuler) {
                     // This must be our original manifold.
                     comp->makeOrphan();
@@ -898,7 +934,7 @@ bool NTriangulation::hasCompressingDisc() const {
 
             if (! comp) {
                 // We must have compressed.
-                return (compressingDisc = true);
+                return (compressingDisc_ = true);
             }
 
             // Around we go again, but this time with a smaller triangulation.
@@ -919,29 +955,29 @@ bool NTriangulation::hasCompressingDisc() const {
             // Use the fact that all vertex normal surfaces are connected.
             if (q->getSurface(i)->isCompressingDisc(true)) {
                 delete use;
-                return (compressingDisc = true);
+                return (compressingDisc_ = true);
             }
         }
 
         // No compressing discs!
         delete use;
-        return (compressingDisc = false);
+        return (compressingDisc_ = false);
     }
 }
 
 bool NTriangulation::knowsCompressingDisc() const {
-    if (compressingDisc.known())
+    if (compressingDisc_.known())
         return true;
 
     // Quickly check for non-spherical boundary components before we give up.
-    for (BoundaryComponentIterator it = boundaryComponents.begin();
-            it != boundaryComponents.end(); ++it) {
-        if ((*it)->getEulerCharacteristic() < 2)
+    for (BoundaryComponentIterator it = boundaryComponents_.begin();
+            it != boundaryComponents_.end(); ++it) {
+        if ((*it)->getEulerChar() < 2)
             return false;
     }
 
     // All boundary components are 2-spheres.
-    compressingDisc = false;
+    compressingDisc_ = false;
     return true;
 }
 
@@ -965,7 +1001,7 @@ bool NTriangulation::hasSimpleCompressingDisc() const {
                 (*cit)->getNumberOfVertices() == 1) {
             // Because we know the triangulation is valid, this rules out
             // all one-tetrahedron triangulations except for LST(1,2,3).
-            return (compressingDisc = true);
+            return (compressingDisc_ = true);
         }
 
     // Open up as many boundary triangles as possible (to make it easier to
@@ -989,7 +1025,7 @@ bool NTriangulation::hasSimpleCompressingDisc() const {
     BoundaryComponentIterator bit;
     for (bit = use.getBoundaryComponents().begin(); bit !=
             use.getBoundaryComponents().end(); ++bit)
-        if ((*bit)->getEulerCharacteristic() == 2)
+        if ((*bit)->getEulerChar() == 2)
             ++origSphereCount;
 
     // Look for a single internal triangle surrounded by three boundary edges.
@@ -1019,17 +1055,17 @@ bool NTriangulation::hasSimpleCompressingDisc() const {
         // non-trivial curve.
         if (cut.getNumberOfBoundaryComponents() ==
                 use.getNumberOfBoundaryComponents())
-            return (compressingDisc = true);
+            return (compressingDisc_ = true);
 
         newSphereCount = 0;
         for (bit = cut.getBoundaryComponents().begin(); bit !=
                 cut.getBoundaryComponents().end(); ++bit)
-            if ((*bit)->getEulerCharacteristic() == 2)
+            if ((*bit)->getEulerChar() == 2)
                 ++newSphereCount;
 
         // Was the boundary of the disc non-trivial?
         if (newSphereCount == origSphereCount)
-            return (compressingDisc = true);
+            return (compressingDisc_ = true);
     }
 
     // Look for a tetrahedron with two faces folded together, giving a
@@ -1038,7 +1074,7 @@ bool NTriangulation::hasSimpleCompressingDisc() const {
     // right through the tetrahedron.
     TetrahedronIterator tit;
     NSnappedBall* ball;
-    for (tit = use.tetrahedra.begin(); tit != use.tetrahedra.end(); ++tit) {
+    for (tit = use.tetrahedra_.begin(); tit != use.tetrahedra_.end(); ++tit) {
         ball = NSnappedBall::formsSnappedBall(*tit);
         if (! ball)
             continue;
@@ -1076,17 +1112,17 @@ bool NTriangulation::hasSimpleCompressingDisc() const {
         // non-trivial curve.
         if (cut.getNumberOfBoundaryComponents() ==
                 use.getNumberOfBoundaryComponents())
-            return (compressingDisc = true);
+            return (compressingDisc_ = true);
 
         newSphereCount = 0;
         for (bit = cut.getBoundaryComponents().begin(); bit !=
                 cut.getBoundaryComponents().end(); ++bit)
-            if ((*bit)->getEulerCharacteristic() == 2)
+            if ((*bit)->getEulerChar() == 2)
                 ++newSphereCount;
 
         // Was the boundary of the disc non-trivial?
         if (newSphereCount == origSphereCount)
-            return (compressingDisc = true);
+            return (compressingDisc_ = true);
     }
 
     // Nothing found.
@@ -1113,8 +1149,8 @@ namespace {
 }
 
 bool NTriangulation::isHaken() const {
-    if (haken.known())
-        return haken.value();
+    if (haken_.known())
+        return haken_.value();
 
     // Check basic preconditions.
     if (! (isValid() && isOrientable() && isClosed() && isConnected()))
@@ -1129,11 +1165,12 @@ bool NTriangulation::isHaken() const {
     // Okay: we are closed, connected, orientable and irreducible.
     // Move to a copy of this triangulation, which we can mess with.
     NTriangulation t(*this);
+    t.intelligentSimplify();
 
     // First check for an easy answer via homology:
     if (t.getHomologyH1().getRank() > 0) {
-        threeSphere = false; // Implied by Hakenness.
-        return (haken = true);
+        threeSphere_ = false; // Implied by Hakenness.
+        return (haken_ = true);
     }
 
     // Enumerate vertex normal surfaces in quad coordinates.
@@ -1147,7 +1184,7 @@ bool NTriangulation::isHaken() const {
     unsigned i;
     for (i = 0; i < list->getNumberOfSurfaces(); ++i) {
         id[i].index = i;
-        id[i].euler = list->getSurface(i)->getEulerCharacteristic().longValue();
+        id[i].euler = list->getSurface(i)->getEulerChar().longValue();
     }
     std::sort(id, id + list->getNumberOfSurfaces());
 
@@ -1157,17 +1194,17 @@ bool NTriangulation::isHaken() const {
         // std::cout << "Testing surface " << i << "..." << std::endl;
         if (list->getSurface(id[i].index)->isIncompressible()) {
             delete[] id;
-            threeSphere = false; // Implied by Hakenness.
-            return (haken = true);
+            threeSphere_ = false; // Implied by Hakenness.
+            return (haken_ = true);
         }
     }
 
     delete[] id;
-    return (haken = false);
+    return (haken_ = false);
 }
 
 bool NTriangulation::knowsHaken() const {
-    return haken.known();
+    return haken_.known();
 }
 
 } // namespace regina
