@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -49,6 +49,7 @@
 
 namespace regina {
 
+class NComponent;
 class NEdge;
 class NTriangle;
 class NTetrahedron;
@@ -62,7 +63,7 @@ class NVertex;
 /**
  * Represents a component of the boundary of a triangulation.
  * Note that an ideal vertex constitutes a boundary component of its
- * own. 
+ * own.
  *
  * We can run into some interesting cases with invalid triangulations.
  * Suppose some vertex link is a multiply punctured surface (which makes
@@ -84,14 +85,14 @@ class NVertex;
 class REGINA_API NBoundaryComponent :
         public ShareableObject, public NMarkedElement {
     private:
-        std::vector<NTriangle*> triangles;
+        std::vector<NTriangle*> triangles_;
             /**< List of triangles in the component. */
-        std::vector<NEdge*> edges;
+        std::vector<NEdge*> edges_;
             /**< List of edges in the component. */
-        std::vector<NVertex*> vertices;
+        std::vector<NVertex*> vertices_;
             /**< List of vertices in the component. */
 
-        bool orientable;
+        bool orientable_;
             /**< Is this boundary component orientable? */
 
     public:
@@ -100,6 +101,15 @@ class REGINA_API NBoundaryComponent :
          */
         virtual ~NBoundaryComponent();
 
+        /**
+         * Returns the index of this boundary component in the underlying
+         * triangulation.  This is identical to calling
+         * <tt>getTriangulation()->boundaryComponentIndex(this)</tt>.
+         *
+         * @return the index of this boundary component vertex.
+         */
+        unsigned long index() const;
+
         /**
          * Returns the number of triangles in this boundary component.
          *
@@ -198,6 +208,14 @@ class REGINA_API NBoundaryComponent :
          */
         NVertex* getVertex(unsigned long index) const;
 
+        /**
+         * Returns the component of the triangulation to which this
+         * boundary component belongs.
+         *
+         * @return the component containing this boundary component.
+         */
+        NComponent* getComponent() const;
+
         /**
          * Returns the Euler characteristic of this boundary component.
          * If this boundary component is ideal, the Euler characteristic
@@ -205,6 +223,20 @@ class REGINA_API NBoundaryComponent :
          *
          * @return the Euler characteristic.
          */
+        long getEulerChar() const;
+
+        /**
+         * A deprecated alias for getEulerChar().
+         *
+         * Returns the Euler characteristic of this boundary component.
+         * If this boundary component is ideal, the Euler characteristic
+         * of the link of the corresponding ideal vertex is returned.
+         *
+         * \deprecated This routine will be removed in a future version of
+         * Regina.  Please use the identical routine getEulerChar() instead.
+         *
+         * @return the Euler characteristic.
+         */
         long getEulerCharacteristic() const;
 
         /**
@@ -219,7 +251,7 @@ class REGINA_API NBoundaryComponent :
 
         /**
          * Determines if this boundary component is orientable.
-         * If the boundary component is ideal, the orientability 
+         * If the boundary component is ideal, the orientability
          * of the link of the corresponding ideal vertex is returned.
          *
          * @return \c true if and only if this boundary component is
@@ -264,58 +296,71 @@ inline NBoundaryComponent::NBoundaryComponent() {
 }
 
 inline NBoundaryComponent::NBoundaryComponent(NVertex* idealVertex) {
-    vertices.push_back(idealVertex);
+    vertices_.push_back(idealVertex);
 }
 
 inline NBoundaryComponent::~NBoundaryComponent() {
 }
 
+inline unsigned long NBoundaryComponent::index() const {
+    return markedIndex();
+}
+
 inline unsigned long NBoundaryComponent::getNumberOfTriangles() const {
-    return triangles.size();
+    return triangles_.size();
 }
 
 inline unsigned long NBoundaryComponent::getNumberOfFaces() const {
-    return triangles.size();
+    return triangles_.size();
 }
 
 inline unsigned long NBoundaryComponent::getNumberOfEdges() const {
-    return edges.size();
+    return edges_.size();
 }
 
 inline unsigned long NBoundaryComponent::getNumberOfVertices() const {
-    return vertices.size();
+    return vertices_.size();
 }
 
 inline NTriangle* NBoundaryComponent::getTriangle(unsigned long index) const {
-    return triangles[index];
+    return triangles_[index];
 }
 
 inline NTriangle* NBoundaryComponent::getFace(unsigned long index) const {
-    return triangles[index];
+    return triangles_[index];
 }
 
 inline NEdge* NBoundaryComponent::getEdge(unsigned long index) const {
-    return edges[index];
+    return edges_[index];
 }
 
 inline NVertex* NBoundaryComponent::getVertex(unsigned long index) const {
-    return vertices[index];
+    return vertices_[index];
+}
+
+inline NComponent* NBoundaryComponent::getComponent() const {
+    // There may be no triangles, but there is always a vertex.
+    return vertices_.front()->getComponent();
+}
+
+inline long NBoundaryComponent::getEulerChar() const {
+    return (isIdeal() ?
+        vertices_.front()->getLinkEulerChar() :
+        long(vertices_.size()) - long(edges_.size()) + long(triangles_.size()));
 }
 
 inline long NBoundaryComponent::getEulerCharacteristic() const {
-    return (isIdeal() ?
-        vertices.front()->getLinkEulerCharacteristic() :
-        long(vertices.size()) - long(edges.size()) + long(triangles.size()));
+    return getEulerChar();
 }
 
 inline bool NBoundaryComponent::isIdeal() const {
-    return triangles.empty();
+    return triangles_.empty();
 }
 
 inline bool NBoundaryComponent::isOrientable() const {
-    return orientable;
+    return orientable_;
 }
-        
+
 inline void NBoundaryComponent::writeTextShort(std::ostream& out) const {
     out << (isIdeal() ? "Ideal " : "Finite ") << "boundary component";
 }
