@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -77,7 +77,7 @@ void NEdge::writeTextLong(std::ostream& out) const {
 
     out << "Appears as:" << std::endl;
     std::deque<NEdgeEmbedding>::const_iterator it;
-    for (it = embeddings.begin(); it != embeddings.end(); ++it)
+    for (it = embeddings_.begin(); it != embeddings_.end(); ++it)
         out << "  " << it->getTetrahedron()->markedIndex()
             << " (" << it->getVertices().trunc2() << ')' << std::endl;
 }
