@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -334,6 +334,64 @@ bool NTriangulation::twoThreeMove(NTriangle* f, bool check, bool perform) {
     return true;
 }
 
+bool NTriangulation::oneFourMove(NTetrahedron* tet, bool /* check */,
+        bool perform) {
+    if ( !perform )
+        return true; // You can always do this move.
+
+    ChangeEventSpan span(this);
+
+    // Before we unglue, record how the adjacent tetrahedra are glued to tet.
+    NTetrahedron* adjTet[4];
+    NPerm4 adjGlue[4];
+    unsigned i, j;
+    for (i=0; i<4; i++) {
+        adjTet[i] = tet->adjacentTetrahedron(i);
+        if (adjTet[i])
+            adjGlue[i] = tet->adjacentGluing(i);
+    }
+
+    // Unglue the old tetrahedron.
+    tet->isolate();
+
+    // The new tetrahedra.
+    // Face i of the old tetrahedron will become a facet of newTet[i].
+    // Vertex i of newTet[i] will become the new internal vertex, and
+    // the other three vertices of newTet[i] will keep the same vertex numbers
+    // that they had in the old tetrahedron.
+    NTetrahedron* newTet[4];
+    for (i = 0; i < 4; ++i)
+        newTet[i] = newTetrahedron();
+
+    // Glue the new tetrahedra to each other internally.
+    for (i = 0; i < 4; ++i)
+        for (j = i + 1; j < 4; ++j)
+            newTet[i]->joinTo(j, newTet[j], NPerm4(i, j));
+
+    // Attach the new tetrahedra to the old triangulation.
+    for (i = 0; i < 4; ++i) {
+        if (adjTet[i] == tet) {
+            // The old tetrahedron was glued to itself.
+
+            // We might have already made this gluing from the other side:
+            if (newTet[i]->adjacentTetrahedron(i))
+                continue;
+
+            // Nope, do it now.
+            newTet[i]->joinTo(i, newTet[adjGlue[i][i]], adjGlue[i]);
+        } else if (adjTet[i]) {
+            // The old tetrahedron was glued elsewhere.
+            newTet[i]->joinTo(i, adjTet[i], adjGlue[i]);
+        }
+    }
+
+    // Delete the old tetrahedron.
+    removeTetrahedron(tet);
+
+    // All done!
+    return true;
+}
+
 bool NTriangulation::fourFourMove(NEdge* e, int newAxis, bool check,
         bool perform) {
     const std::deque<NEdgeEmbedding>& embs = e->getEmbeddings();
@@ -783,7 +841,7 @@ bool NTriangulation::shellBoundary(NTetrahedron* t,
         bool check, bool perform) {
     // To perform the move we don't even need a skeleton.
     if (check) {
-        if (! calculatedSkeleton)
+        if (! calculatedSkeleton_)
             calculateSkeleton();
 
         int nBdry = 0;
@@ -932,7 +990,7 @@ bool NTriangulation::collapseEdge(NEdge* e, bool check, bool perform) {
                 return false;
 
         {
-            long nEdges = edges.size();
+            long nEdges = edges_.size();
 
             // The parent of each edge in the union-find tree, or -1 if
             // an edge is at the root of a tree.
@@ -1010,7 +1068,7 @@ bool NTriangulation::collapseEdge(NEdge* e, bool check, bool perform) {
         // overkill, since each vertex in the corresponding graph G will
         // have degree <= 2, but it's fast so we'll do it.
         {
-            long nTriangles = triangles.size();
+            long nTriangles = triangles_.size();
 
             // The parent of each triangle in the union-find tree, or -1 if
             // a triangle is at the root of a tree.
@@ -1120,7 +1178,7 @@ void NTriangulation::reorderTetrahedraBFS(bool reverse) {
             while (used[nextTet])
                 ++nextTet;
 
-            ordered[filled++] = tetrahedra[nextTet];
+            ordered[filled++] = tetrahedra_[nextTet];
             used[nextTet] = true;
             ++nextTet;
         }
@@ -1140,15 +1198,15 @@ void NTriangulation::reorderTetrahedraBFS(bool reverse) {
 
     // Flush the tetrahedra from the triangulation, and reinsert them in
     // the order in which they were found during the breadth-first search.
-    tetrahedra.clear();
+    tetrahedra_.clear();
 
     unsigned long j;
     if (reverse) {
         for (j = n; j > 0; )
-            tetrahedra.push_back(ordered[--j]);
+            tetrahedra_.push_back(ordered[--j]);
     } else {
         for (j = 0; j < n; )
-            tetrahedra.push_back(ordered[j++]);
+            tetrahedra_.push_back(ordered[j++]);
     }
 
     delete[] used;
