@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -61,6 +61,12 @@ namespace {
         NGroupExpression::substitute, 2, 3);
     BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(OL_addGenerator,
         NGroupPresentation::addGenerator, 0, 1);
+    BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(OL_nielsenCombine,
+        NGroupPresentation::nielsenCombine, 3, 4);
+
+    std::auto_ptr<NGroupExpression> newExpression_str(const std::string& str) {
+        return std::auto_ptr<NGroupExpression>(new NGroupExpression(str, 0));
+    }
 
     object getTerms_list(const NGroupExpression& e) {
         boost::python::list ans;
@@ -95,20 +101,6 @@ namespace {
         p.writeTextCompact(std::cout);
     }
 
-    regina::NHomGroupPresentation* intelligentSimplifyDetail_ptr(
-            NGroupPresentation& p) {
-        return p.intelligentSimplifyDetail().release();
-    }
-
-    regina::NAbelianGroup* abelianisation_ptr(const NGroupPresentation& p) {
-        return p.abelianisation().release();
-    }
-
-    regina::NMarkedAbelianGroup* markedAbelianisation_ptr(
-            const NGroupPresentation& p) {
-        return p.markedAbelianisation().release();
-    }
-
     BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(OL_proliferateRelators,
         NGroupPresentation::proliferateRelators, 0, 1);
 }
@@ -120,6 +112,7 @@ void addNGroupPresentation() {
         .def(init<unsigned long, long>())
         .def(init<const NGroupExpressionTerm&>())
         .def(self == self)
+        .def(self < self)
         .def("inverse", &NGroupExpressionTerm::inverse)
         .def(self += self)
         .def(self_ns::str(self))
@@ -129,9 +122,12 @@ void addNGroupPresentation() {
             std::auto_ptr<NGroupExpression>, boost::noncopyable>
             ("NGroupExpression")
         .def(init<const NGroupExpression&>())
+        .def("__init__", boost::python::make_constructor(newExpression_str))
+        .def(self == self)
         .def("getTerms", getTerms_list)
         .def("getNumberOfTerms", &NGroupExpression::getNumberOfTerms)
         .def("wordLength", &NGroupExpression::wordLength)
+        .def("isTrivial", &NGroupExpression::isTrivial)
         .def("erase", &NGroupExpression::erase)
         .def("getTerm", getTerm_non_const, return_internal_reference<>())
         .def("getGenerator", &NGroupExpression::getGenerator)
@@ -142,6 +138,8 @@ void addNGroupPresentation() {
         .def("addTermLast", addTermLast_pair)
         .def("addTermsFirst", &NGroupExpression::addTermsFirst)
         .def("addTermsLast", &NGroupExpression::addTermsLast)
+        .def("addStringFirst", &NGroupExpression::addStringFirst)
+        .def("addStringLast", &NGroupExpression::addStringLast)
         .def("cycleLeft", &NGroupExpression::cycleLeft)
         .def("cycleRight", &NGroupExpression::cycleRight)
         .def("inverse", &NGroupExpression::inverse,
@@ -168,18 +166,36 @@ void addNGroupPresentation() {
         .def("getNumberOfRelations", &NGroupPresentation::getNumberOfRelations)
         .def("getRelation", &NGroupPresentation::getRelation,
             return_internal_reference<>())
+        .def("isValid", &NGroupPresentation::isValid)
         .def("intelligentSimplify", &NGroupPresentation::intelligentSimplify)
         .def("intelligentSimplifyDetail",
-            intelligentSimplifyDetail_ptr,
-            return_value_policy<manage_new_object>())
+            &NGroupPresentation::intelligentSimplifyDetail)
+        .def("smallCancellation", &NGroupPresentation::smallCancellation)
+        .def("smallCancellationDetail",
+            &NGroupPresentation::smallCancellationDetail)
+        .def("simplifyWord", &NGroupPresentation::simplifyWord)
         .def("proliferateRelators", &NGroupPresentation::proliferateRelators,
             OL_proliferateRelators())
+        .def("identifyAbelian", &NGroupPresentation::identifyAbelian)
+        .def("nielsenTransposition", &NGroupPresentation::nielsenTransposition)
+        .def("nielsenInvert", &NGroupPresentation::nielsenInvert)
+        .def("nielsenCombine", &NGroupPresentation::nielsenCombine,
+            OL_nielsenCombine())
+        .def("intelligentNielsen", &NGroupPresentation::intelligentNielsen)
+        .def("intelligentNielsenDetail",
+            &NGroupPresentation::intelligentNielsenDetail)
+        .def("homologicalAlignment", &NGroupPresentation::homologicalAlignment)
+        .def("homologicalAlignmentDetail",
+            &NGroupPresentation::homologicalAlignmentDetail)
+        .def("prettyRewriting", &NGroupPresentation::prettyRewriting)
+        .def("prettyRewritingDetail",
+            &NGroupPresentation::prettyRewritingDetail)
+        .def("identifySimplyIsomorphicTo",
+            &NGroupPresentation::identifySimplyIsomorphicTo)
         .def("recogniseGroup", &NGroupPresentation::recogniseGroup)
         .def("relatorLength", &NGroupPresentation::relatorLength)
-        .def("abelianisation", abelianisation_ptr,
-            return_value_policy<manage_new_object>())
-        .def("markedAbelianisation", markedAbelianisation_ptr,
-            return_value_policy<manage_new_object>())
+        .def("abelianisation", &NGroupPresentation::abelianisation)
+        .def("markedAbelianisation", &NGroupPresentation::markedAbelianisation)
         .def("toTeX", &NGroupPresentation::toTeX)
         .def("toStringCompact", &NGroupPresentation::toStringCompact)
         .def("compact", &NGroupPresentation::compact)
