@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -76,7 +76,10 @@ void addNAngleStructureList() {
             return_value_policy<reference_existing_object>())
         .def("enumerate", enumerate_3,
             return_value_policy<reference_existing_object>())
+        .def("enumerateTautDD", &NAngleStructureList::enumerateTautDD,
+            return_value_policy<reference_existing_object>())
         .staticmethod("enumerate")
+        .staticmethod("enumerateTautDD")
     ;
 
     s.attr("packetType") = regina::PacketType(NAngleStructureList::packetType);
