@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -45,8 +45,11 @@ namespace {
 void addNPDF() {
     scope s = class_<NPDF, bases<regina::NPacket>,
             std::auto_ptr<NPDF>, boost::noncopyable>("NPDF", init<>())
+        .def(init<const char*>())
+        .def("isNull", &NPDF::isNull)
         .def("size", &NPDF::size)
         .def("reset", reset_empty)
+        .def("savePDF", &NPDF::savePDF)
     ;
 
     s.attr("packetType") = regina::PacketType(NPDF::packetType);
