@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -43,20 +43,20 @@ namespace {
         const = &NScript::getVariableValue;
     regina::NPacket* (NScript::*getVariableValue_string)(const std::string&)
         const = &NScript::getVariableValue;
+
+    void (NScript::*removeVariable_long)(unsigned long) =
+        &NScript::removeVariable;
+    void (NScript::*removeVariable_string)(const std::string&) =
+        &NScript::removeVariable;
 }
 
 void addNScript() {
     scope s = class_<NScript, bases<regina::NPacket>,
             std::auto_ptr<NScript>, boost::noncopyable>("NScript", init<>())
-        .def("getNumberOfLines", &NScript::getNumberOfLines)
-        .def("getLine", &NScript::getLine,
+        .def("getText", &NScript::getText,
             return_value_policy<return_by_value>())
-        .def("addFirst", &NScript::addFirst)
-        .def("addLast", &NScript::addLast)
-        .def("insertAtPosition", &NScript::insertAtPosition)
-        .def("replaceAtPosition", &NScript::replaceAtPosition)
-        .def("removeLineAt", &NScript::removeLineAt)
-        .def("removeAllLines", &NScript::removeAllLines)
+        .def("setText", &NScript::setText)
+        .def("append", &NScript::append)
         .def("getNumberOfVariables", &NScript::getNumberOfVariables)
         .def("getVariableName", &NScript::getVariableName,
             return_value_policy<return_by_value>())
@@ -64,8 +64,12 @@ void addNScript() {
             return_value_policy<reference_existing_object>())
         .def("getVariableValue", getVariableValue_string,
             return_value_policy<reference_existing_object>())
+        .def("getVariableIndex", &NScript::getVariableIndex)
+        .def("setVariableName", &NScript::setVariableName)
+        .def("setVariableValue", &NScript::setVariableValue)
         .def("addVariable", &NScript::addVariable)
-        .def("removeVariable", &NScript::removeVariable)
+        .def("removeVariable", removeVariable_long)
+        .def("removeVariable", removeVariable_string)
         .def("removeAllVariables", &NScript::removeAllVariables)
     ;
 
