@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -160,6 +160,7 @@ void addNNormalSurface() {
         .def("writeRawVector", writeRawVector_stdio)
         .def("isEmpty", &NNormalSurface::isEmpty)
         .def("isCompact", &NNormalSurface::isCompact)
+        .def("getEulerChar", &NNormalSurface::getEulerChar)
         .def("getEulerCharacteristic", &NNormalSurface::getEulerCharacteristic)
         .def("isOrientable", &NNormalSurface::isOrientable)
         .def("isTwoSided", &NNormalSurface::isTwoSided)
@@ -182,7 +183,7 @@ void addNNormalSurface() {
         .def("locallyCompatible", &NNormalSurface::locallyCompatible)
         .def("disjoint", &NNormalSurface::disjoint)
 #ifndef EXCLUDE_NORMALIZ
-        .def("boundarySlopes", &NNormalSurface::boundarySlopes,
+        .def("boundaryIntersections", &NNormalSurface::boundaryIntersections,
             return_value_policy<manage_new_object>())
 #endif
         .def("findNonTrivialSphere", &NNormalSurface::findNonTrivialSphere,
