@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -81,10 +81,13 @@ void addNEdge() {
             return_value_policy<reference_existing_object>())
         .def("getEdge", &NEdgeEmbedding::getEdge)
         .def("getVertices", &NEdgeEmbedding::getVertices)
+        .def(self == self)
+        .def(self != self)
     ;
 
     scope s = class_<NEdge, bases<regina::ShareableObject>,
             std::auto_ptr<NEdge>, boost::noncopyable>("NEdge", no_init)
+        .def("index", &NEdge::index)
         .def("getEmbeddings", edge_getEmbeddings_list)
         .def("getNumberOfEmbeddings", &NEdge::getNumberOfEmbeddings)
         .def("getEmbedding", &NEdge::getEmbedding,
