@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -71,6 +71,8 @@ void addNExampleTriangulation() {
         .def("smallClosedNonOrblHyperbolic",
             &NExampleTriangulation::smallClosedNonOrblHyperbolic,
             return_value_policy<manage_new_object>())
+        .def("sphere600", &NExampleTriangulation::sphere600,
+            return_value_policy<manage_new_object>())
         .def("lst3_4_7", &NExampleTriangulation::lst3_4_7,
             return_value_policy<manage_new_object>())
         .def("solidKleinBottle", &NExampleTriangulation::solidKleinBottle,
@@ -78,6 +80,9 @@ void addNExampleTriangulation() {
         .def("figureEightKnotComplement",
             &NExampleTriangulation::figureEightKnotComplement,
             return_value_policy<manage_new_object>())
+        .def("trefoilKnotComplement",
+            &NExampleTriangulation::trefoilKnotComplement,
+            return_value_policy<manage_new_object>())
         .def("whiteheadLinkComplement",
             &NExampleTriangulation::whiteheadLinkComplement,
             return_value_policy<manage_new_object>())
@@ -98,9 +103,11 @@ void addNExampleTriangulation() {
         .staticmethod("weberSeifert")
         .staticmethod("smallClosedOrblHyperbolic")
         .staticmethod("smallClosedNonOrblHyperbolic")
+        .staticmethod("sphere600")
         .staticmethod("lst3_4_7")
         .staticmethod("solidKleinBottle")
         .staticmethod("figureEightKnotComplement")
+        .staticmethod("trefoilKnotComplement")
         .staticmethod("whiteheadLinkComplement")
         .staticmethod("gieseking")
         .staticmethod("cuspedGenusTwoTorus")
