@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -34,6 +34,7 @@
 
 #include <boost/python.hpp>
 #include "algebra/ngrouppresentation.h"
+#include "angle/nanglestructure.h"
 #include "surfaces/nnormalsurface.h"
 #include "triangulation/nisomorphism.h"
 #include "triangulation/ntriangulation.h"
@@ -52,6 +53,12 @@ namespace {
         &NTriangulation::twoZeroMove;
     bool (NTriangulation::*twoZeroMove_edge)(regina::NEdge*, bool, bool) =
         &NTriangulation::twoZeroMove;
+    std::string (NTriangulation::*snapPea_void)() const =
+        &NTriangulation::snapPea;
+    std::string (NTriangulation::*recogniser_void)() const =
+        &NTriangulation::recogniser;
+    std::string (NTriangulation::*recognizer_void)() const =
+        &NTriangulation::recognizer;
 
     BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(OL_simplifyToLocalMinimum,
         NTriangulation::simplifyToLocalMinimum, 0, 1);
@@ -59,6 +66,8 @@ namespace {
         NTriangulation::threeTwoMove, 1, 3);
     BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(OL_twoThreeMove,
         NTriangulation::twoThreeMove, 1, 3);
+    BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(OL_oneFourMove,
+        NTriangulation::oneFourMove, 1, 3);
     BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(OL_fourFourMove,
         NTriangulation::fourFourMove, 2, 4);
     BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(OL_twoZeroMove,
@@ -83,6 +92,8 @@ namespace {
         NTriangulation::connectedSumDecomposition, 0, 2);
     BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(OL_idealToFinite,
         NTriangulation::idealToFinite, 0, 1);
+    BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(OL_puncture,
+        NTriangulation::puncture, 0, 1);
 
     void simplifiedFundamentalGroup_own(NTriangulation& tri,
             std::auto_ptr<regina::NGroupPresentation> group) {
@@ -99,16 +110,6 @@ namespace {
         tet.release();
     }
 
-    regina::NIsomorphism* isIsomorphicTo_ptr(NTriangulation& t,
-            NTriangulation& s) {
-        return t.isIsomorphicTo(s).release();
-    }
-
-    regina::NIsomorphism* isContainedIn_ptr(NTriangulation& t,
-            NTriangulation& s) {
-        return t.isContainedIn(s).release();
-    }
-
     boost::python::list getTetrahedra_list(NTriangulation& t) {
         boost::python::list ans;
         for (NTriangulation::TetrahedronIterator it =
@@ -158,6 +159,22 @@ namespace {
         return ans;
     }
 
+    boost::python::list maximalForestInDualSkeleton_list(NTriangulation& t) {
+        std::set<regina::NTriangle*> triangleSet;
+        t.maximalForestInDualSkeleton(triangleSet);
+
+        // boost::python does not contain python sets. We use a python list
+        // here instead which needs a well-defined order, thus we iterate
+        // through getTriangles().
+
+        boost::python::list ans;
+        for (NTriangulation::TriangleIterator it =
+                t.getTriangles().begin(); it != t.getTriangles().end(); it++)
+            if (triangleSet.count(*it) > 0)
+                ans.append(boost::python::ptr(*it));
+        return ans;
+    }
+
     std::string isoSig_void(const NTriangulation& t) {
         return t.isoSig();
     }
@@ -239,14 +256,15 @@ void addNTriangulation() {
         .def("edgeIndex", &NTriangulation::edgeIndex)
         .def("faceIndex", &NTriangulation::faceIndex)
         .def("triangleIndex", &NTriangulation::triangleIndex)
-        .def("isIsomorphicTo", isIsomorphicTo_ptr,
-            return_value_policy<manage_new_object>())
-        .def("isContainedIn", isContainedIn_ptr,
-            return_value_policy<manage_new_object>())
+        .def("isIdenticalTo", &NTriangulation::isIdenticalTo)
+        .def("isIsomorphicTo", &NTriangulation::isIsomorphicTo)
+        .def("makeCanonical", &NTriangulation::makeCanonical)
+        .def("isContainedIn", &NTriangulation::isContainedIn)
         .def("hasTwoSphereBoundaryComponents",
             &NTriangulation::hasTwoSphereBoundaryComponents)
         .def("hasNegativeIdealBoundaryComponents",
             &NTriangulation::hasNegativeIdealBoundaryComponents)
+        .def("isEmpty", &NTriangulation::isEmpty)
         .def("getEulerCharTri", &NTriangulation::getEulerCharTri)
         .def("getEulerCharManifold", &NTriangulation::getEulerCharManifold)
         .def("getEulerCharacteristic", &NTriangulation::getEulerCharacteristic)
@@ -255,6 +273,8 @@ void addNTriangulation() {
         .def("isStandard", &NTriangulation::isStandard)
         .def("hasBoundaryFaces", &NTriangulation::hasBoundaryFaces)
         .def("hasBoundaryTriangles", &NTriangulation::hasBoundaryTriangles)
+        .def("getNumberOfBoundaryTriangles",
+            &NTriangulation::getNumberOfBoundaryTriangles)
         .def("isClosed", &NTriangulation::isClosed)
         .def("isOrientable", &NTriangulation::isOrientable)
         .def("isOriented", &NTriangulation::isOriented)
@@ -283,11 +303,20 @@ void addNTriangulation() {
         .def("hasOctagonalAlmostNormalSphere",
             &NTriangulation::hasOctagonalAlmostNormalSphere,
             return_value_policy<manage_new_object>())
+        .def("findStrictAngleStructure",
+            &NTriangulation::findStrictAngleStructure,
+            return_internal_reference<>())
+        .def("hasStrictAngleStructure",
+            &NTriangulation::hasStrictAngleStructure)
+        .def("knowsStrictAngleStructure",
+            &NTriangulation::knowsStrictAngleStructure)
+        .def("maximalForestInDualSkeleton", maximalForestInDualSkeleton_list)
         .def("intelligentSimplify", &NTriangulation::intelligentSimplify)
         .def("simplifyToLocalMinimum", &NTriangulation::simplifyToLocalMinimum,
             OL_simplifyToLocalMinimum())
         .def("threeTwoMove", &NTriangulation::threeTwoMove, OL_threeTwoMove())
         .def("twoThreeMove", &NTriangulation::twoThreeMove, OL_twoThreeMove())
+        .def("oneFourMove", &NTriangulation::oneFourMove, OL_oneFourMove())
         .def("fourFourMove", &NTriangulation::fourFourMove, OL_fourFourMove())
         .def("twoZeroMove", twoZeroMove_vertex, OL_twoZeroMove())
         .def("twoZeroMove", twoZeroMove_edge, OL_twoZeroMove())
@@ -326,6 +355,7 @@ void addNTriangulation() {
         .def("finiteToIdeal", &NTriangulation::finiteToIdeal)
         .def("barycentricSubdivision", &NTriangulation::barycentricSubdivision)
         .def("drillEdge", &NTriangulation::drillEdge)
+        .def("puncture", &NTriangulation::puncture, OL_puncture())
         .def("layerOn", &NTriangulation::layerOn,
             return_value_policy<reference_existing_object>())
         .def("insertLayeredSolidTorus",
@@ -336,6 +366,7 @@ void addNTriangulation() {
         .def("insertLayeredLoop", &NTriangulation::insertLayeredLoop)
         .def("insertAugTriSolidTorus", &NTriangulation::insertAugTriSolidTorus)
         .def("insertSFSOverSphere", &NTriangulation::insertSFSOverSphere)
+        .def("connectedSumWith", &NTriangulation::connectedSumWith)
         .def("insertTriangulation", &NTriangulation::insertTriangulation)
         .def("insertRehydration", &NTriangulation::insertRehydration)
         .def("dehydrate", &NTriangulation::dehydrate)
@@ -347,7 +378,12 @@ void addNTriangulation() {
             return_value_policy<manage_new_object>())
         .def("isoSigComponentSize", &NTriangulation::isoSigComponentSize)
         .def("dumpConstruction", &NTriangulation::dumpConstruction)
-        .def("snapPea", &NTriangulation::snapPea)
+        .def("snapPea", snapPea_void)
+        .def("saveSnapPea", &NTriangulation::saveSnapPea)
+        .def("recogniser", recogniser_void)
+        .def("recognizer", recognizer_void)
+        .def("saveRecogniser", &NTriangulation::saveRecogniser)
+        .def("saveRecognizer", &NTriangulation::saveRecognizer)
         .def("fromSnapPea", &NTriangulation::fromSnapPea,
             return_value_policy<manage_new_object>())
         .def("enterTextTriangulation", enterTextTriangulation_stdio,
