@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -73,8 +73,7 @@ bool PDFHandler::exportData(regina::NPacket* data, const QString& fileName,
                 "real PDF data."));
         return false;
     }
-    if (! regina::writePDF(
-            static_cast<const char*>(QFile::encodeName(fileName)), *pdf)) {
+    if (! pdf->savePDF(static_cast<const char*>(QFile::encodeName(fileName)))) {
         ReginaSupport::warn(parentWidget,
             QObject::tr("The export failed."), 
             QObject::tr("<qt>An unknown error occurred, probably related "
