@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -48,7 +48,7 @@
 const SourceHandler SourceHandler::instance;
 
 PacketFilter* SourceHandler::canExport() const {
-    return new SingleTypeFilter<regina::NTriangulation>();
+    return new SubclassFilter<regina::NTriangulation>();
 }
 
 bool SourceHandler::exportData(regina::NPacket* data, const QString& fileName,
