@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -77,7 +77,8 @@ QString NAngleStructureCreator::parentWhatsThis() {
 
 regina::NPacket* NAngleStructureCreator::createPacket(
         regina::NPacket* parentPacket, QWidget* parentWidget) {
-    if (parentPacket->getPacketType() != regina::NTriangulation::packetType) {
+    // Note that parent may be either NTriangulation or NSnapPeaTriangulation.
+    if (! dynamic_cast<regina::NTriangulation*>(parentPacket)) {
         ReginaSupport::sorry(ui,
             ui->tr("The selected parent is not a 3-manifold triangulation."),
             ui->tr("Angle structures must live within a 3-manifold "
