@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -230,9 +230,6 @@ void NAngleStructureUI::refresh() {
 
     // Rebuild the table.
     model->rebuild();
-
-    // Tidy up.
-    setDirty(false);
 }
 
 void NAngleStructureUI::refreshHeader() {
