@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -72,9 +72,8 @@ void NPDFExternalViewer::view(regina::NPacket* packet, QWidget* parentWidget) {
         return;
     }
 
-    if (! regina::writePDF(static_cast<const char*>(
-            QFile::encodeName(temp->localFileName())),
-            static_cast<regina::NPDF&>(*packet))) {
+    if (! static_cast<regina::NPDF*>(packet)->savePDF(static_cast<const char*>(
+            QFile::encodeName(temp->localFileName())))) {
         ReginaSupport::warn(parentWidget,
             QObject::tr("<qt>An error occurred whilst writing the PDF "
             "data to the temporary file <i>%1</i>.</qt>").
