@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -73,11 +73,6 @@ class SurfaceModel : public QAbstractItemModel {
         unsigned* realIndex;
         unsigned nFiltered;
 
-        /**
-         * Local modifications
-         */
-        QString* localName;
-
         /**
          * Internal status
          */
@@ -104,12 +99,6 @@ class SurfaceModel : public QAbstractItemModel {
         void rebuild(regina::NormalCoords coordSystem_,
             regina::NSurfaceFilter* filter);
 
-        /**
-         * Loading and saving local data from/to the packet.
-         */
-        void refreshNames();
-        void commitNames();
-
         /**
          * Updating read/write status.
          */
@@ -183,12 +172,11 @@ class NSurfaceCoordinateUI : public QObject, public PacketEditorTab,
 
         /**
          * PacketEditorTab overrides.
+         * Note that refresh() is a slot now.
          */
         regina::NPacket* getPacket();
         QWidget* getInterface();
         const QLinkedList<QAction*>& getPacketTypeActions();
-        void commit();
-        void refresh();
         void setReadWrite(bool readWrite);
 
         /**
@@ -198,10 +186,9 @@ class NSurfaceCoordinateUI : public QObject, public PacketEditorTab,
 
     public slots:
         /**
-         * Refill the table of surfaces whilst preserving local
-         * modifications.
+         * More PacketEditorTab overrides.
          */
-        void refreshLocal();
+        void refresh();
 
         /**
          * Surface list actions.
@@ -218,16 +205,10 @@ class NSurfaceCoordinateUI : public QObject, public PacketEditorTab,
          * Provides auto-resizing of columns.
          */
         void columnResized(int section, int oldSize, int newSize);
-
-        /**
-         * Notify us that a surface has been renamed.
-         */
-        void notifySurfaceRenamed();
 };
 
 inline SurfaceModel::~SurfaceModel() {
     delete[] realIndex;
-    delete[] localName;
 }
 
 inline regina::NNormalSurfaceList* SurfaceModel::surfaces() const {
