@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -129,10 +129,6 @@ void NTriHeaderUI::refresh() {
     header->setText(summaryInfo(tri));
 }
 
-void NTriHeaderUI::editingElsewhere() {
-    header->setText(QObject::tr("Editing..."));
-}
-
 QString NTriHeaderUI::summaryInfo(regina::NTriangulation* tri) {
     if (tri->getNumberOfTetrahedra() == 0)
         return QObject::tr("Empty");
