@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -117,7 +117,7 @@ NTriCompositionUI::NTriCompositionUI(regina::NTriangulation* packet,
     label->setWhatsThis(msg);
     isoSelectArea->addWidget(label);
     isoTest = new PacketChooser(tri->getTreeMatriarch(),
-        new SingleTypeFilter<NTriangulation>(),
+        new SubclassFilter<NTriangulation>(),
         PacketChooser::ROOT_AS_PACKET, true, 0, ui);
     isoTest->setAutoUpdate(true);
     isoTest->setWhatsThis(msg);
@@ -247,12 +247,6 @@ void NTriCompositionUI::refresh() {
     details->setRootIsDecorated(foundInnerChildren);
 }
 
-void NTriCompositionUI::editingElsewhere() {
-    details->clear();
-    (new QTreeWidgetItem(details))->setText(0,tr("Editing..."));
-    details->setRootIsDecorated(false);
-}
-
 void NTriCompositionUI::packetToBeDestroyed(regina::NPacket*) {
     // Our current isomorphism test triangulation is about to be
     // destroyed.
