@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -209,19 +209,6 @@ void NTriSkelCompUI::refresh() {
         (it.next())->refresh();
 }
 
-void NTriSkelCompUI::editingElsewhere() {
-    nVertices->setText(tr("Editing..."));
-    nEdges->setText(tr("Editing..."));
-    nTriangles->setText(tr("Editing..."));
-    nTets->setText(tr("Editing..."));
-    nComps->setText(tr("Editing..."));
-    nBdryComps->setText(tr("Editing..."));
-
-    QLinkedListIterator<SkeletonWindow*> it(viewers);
-    while( it.hasNext())
-        (it.next())->editingElsewhere();
-}
-
 void NTriSkelCompUI::viewVertices() {
     // Because we pass this as parent to the new window, we are
     // guaranteed that the window will be closed and deleted
