@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -67,8 +67,7 @@ class NTriSnapPeaUI : public QObject, public PacketViewerTab {
         /**
          * Internal components
          */
-        QWidget* ui;
-        QStackedWidget* data;
+        QStackedWidget* ui;
         QWidget* dataValid;
         QWidget* dataNull;
         QLabel* solutionType;
@@ -91,7 +90,6 @@ class NTriSnapPeaUI : public QObject, public PacketViewerTab {
         regina::NPacket* getPacket();
         QWidget* getInterface();
         void refresh();
-        void editingElsewhere();
 
     public slots:
         /**
@@ -99,6 +97,11 @@ class NTriSnapPeaUI : public QObject, public PacketViewerTab {
          */
         void updatePreferences();
 
+        /**
+         * Convert this to a full SnapPea triangulation.
+         */
+        void toSnapPea();
+
     private:
         /**
          * These routines convert an NSnapPeaTriangulation::SolutionType
