@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -66,14 +66,23 @@ class NTriSurfacesUI : public QObject, public PacketViewerTab {
          */
         QWidget* ui;
         QLabel* zeroEff;
+        QLabel* titleZeroEff;
         QLabel* splitting;
+        QLabel* titleSplitting;
         QLabel* threeSphere;
+        QLabel* titleThreeSphere;
         QLabel* threeBall;
+        QLabel* titleThreeBall;
         QLabel* solidTorus;
+        QLabel* titleSolidTorus;
         QLabel* irreducible;
         QLabel* titleIrreducible;
         QLabel* haken;
         QLabel* titleHaken;
+        QLabel* strict;
+        QLabel* titleStrict;
+        QLabel* hyperbolic;
+        QLabel* titleHyperbolic;
         QAbstractButton* btnZeroEff;
         QAbstractButton* btnSplitting;
         QAbstractButton* btnThreeSphere;
@@ -81,7 +90,9 @@ class NTriSurfacesUI : public QObject, public PacketViewerTab {
         QAbstractButton* btnSolidTorus;
         QAbstractButton* btnIrreducible;
         QAbstractButton* btnHaken;
+        QAbstractButton* btnStrict;
         QLabel* manifold;
+        QLabel* census;
 
     public:
         /**
@@ -96,7 +107,6 @@ class NTriSurfacesUI : public QObject, public PacketViewerTab {
         regina::NPacket* getPacket();
         QWidget* getInterface();
         void refresh();
-        void editingElsewhere();
 
     public slots:
         /**
@@ -109,6 +119,7 @@ class NTriSurfacesUI : public QObject, public PacketViewerTab {
         void calculateSolidTorus();
         void calculateIrreducible();
         void calculateHaken();
+        void calculateStrict();
 
         /**
          * Notify that preferences have changed.
