@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -144,11 +144,6 @@ void SkeletonWindow::refresh() {
     table->header()->resizeSections(QHeaderView::ResizeToContents);
 }
 
-void SkeletonWindow::editingElsewhere() {
-    setWindowTitle(tr("Editing triangulation..."));
-    model->makeEmpty();
-}
-
 void SkeletonWindow::packetWasChanged(regina::NPacket*) {
     refresh();
 }
@@ -205,10 +200,10 @@ QVariant VertexModel::data(const QModelIndex& index, int role) const {
                 if (link == NVertex::NON_STANDARD_CUSP) {
                     if (item->isLinkOrientable())
                         return tr("Cusp (orbl, genus %1)").arg(
-                            1 - (item->getLinkEulerCharacteristic() / 2));
+                            1 - (item->getLinkEulerChar() / 2));
                     else
                         return tr("Cusp (non-or, genus %1)").arg(
-                            2 - item->getLinkEulerCharacteristic());
+                            2 - item->getLinkEulerChar());
                 }
                 if (link == NVertex::NON_STANDARD_BDRY)
                     return tr("Non-std bdry");
