@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -67,12 +67,6 @@ namespace regina {
  * user interaction.  The PacketManager class is responsible for
  * creating an interface component appropriate for a given packet.
  *
- * Subclasses should call setDirty(true) whenever changes are made in
- * the interface.  Likewise, they should call setDirty(false) at the
- * end of their implementations of commit() and refresh().
- * Changes must never be made to the underlying packet except for within
- * the commit() routine.
- *
  * Each packet interface is either in read-write mode or in read-only
  * mode, and may be required to change modes throughout its life span.
  * See setReadWrite() for details.
@@ -135,23 +129,6 @@ class PacketUI {
          */
         virtual QString getPacketMenuText() const = 0;
 
-        /**
-         * Store any changes currently made in this interface in the
-         * underlying packet.
-         *
-         * Note that if a packet interface wishes to force a commit, it
-         * should call enclosingPane->commit() and not this routine.
-         * This will ensure that the enclosing pane can keep up to date
-         * with what is taking place.
-         *
-         * No checking needs to be done as to whether the commit should
-         * be allowed; this is taken care of by PacketPane::commit().
-         *
-         * This routine should call setDirty(false) once changes have
-         * been made.
-         */
-        virtual void commit() = 0;
-
         /**
          * Update this interface to reflect the current contents of the
          * underlying packet.
@@ -160,12 +137,54 @@ class PacketUI {
          * should call enclosingPane->refresh() and not this routine.
          * This will ensure that the enclosing pane can keep up to date
          * with what is taking place.
-         *
-         * This routine should call setDirty(false) once the interface
-         * has been updated.
          */
         virtual void refresh() = 0;
 
+        /**
+         * Ensures that any edits-in-progress are fully committed to the
+         * calculation engine.
+         *
+         * For many interface elements, edits are automatically committed
+         * before anything else is done with the packet (in particular,
+         * before an action is triggered on the packet).  However, there
+         * are exceptions.  For instance:
+         *
+         * - If a QTableView or QListView cell is being edited and an
+         *   action is triggered, the table or list may not update
+         *   its model (i.e., commit the edit) until \e after the action
+         *   event has been processed.  This means that the action event
+         *   needs to force the commit itself, to avoid operating on the
+         *   old version of the packet.
+         *
+         * - If a DocWidget text document is being edited, edits are
+         *   automatically comitted only when the widget loses focus.
+         *   As before, if an action is triggered then the focus-out
+         *   event (and hence the commit) might not be processed until
+         *   after the action.
+         *
+         * Any packet interface that suffers from such problems must do
+         * two things:
+         *
+         * - override endEdit() to commit any edits-in-progress to the
+         *   calculation engine (the table class EditTableView, the list
+         *   class EditListView and the text document class DocWidget
+         *   provide convenient means for doing this);
+         *
+         * - call endEdit() as the first step for each action event on
+         *   the packet (e.g., each action slot).
+         *
+         * It is possible that an edit-in-progress cannot be committed
+         * (for instance, the table cell being edited contains invalid data).
+         * In this case an appropriate error message may be displayed
+         * to the user, and the edit should be rolled back.
+         *
+         * The default implementation of this routine does nothing
+         * (i.e., it assumes there are no edits-in-progress to commit,
+         * or else the commits are (i) automatic and (ii) happen before
+         * any actions are triggered).
+         */
+        virtual void endEdit();
+
         /**
          * Modify this interface to be read-write or read-only according
          * to the given argument.
@@ -179,22 +198,6 @@ class PacketUI {
          */
         virtual void setReadWrite(bool readWrite) = 0;
 
-    protected:
-        /**
-         * Notifies external interface elements that this interface does
-         * (or does not) contain changes that have not yet been committed
-         * to the underlying packet.
-         *
-         * This routine must be called whenever changes are made in the
-         * interface, and must also be called at the end of the
-         * implementations of commit() and refresh().
-         *
-         * This routine should generally not be overridden by
-         * subclasses.  If it is however, any reimplementations must
-         * call the parent implementation.
-         */
-        virtual void setDirty(bool newDirty);
-
     private:
         /**
          * An empty action list.
@@ -215,12 +218,6 @@ class PacketReadOnlyUI : public PacketUI {
          */
         PacketReadOnlyUI(PacketPane* newEnclosingPane);
 
-        /**
-         * An implementation of commit() that does nothing but call
-         * setDirty(false).
-         */
-        virtual void commit();
-
         /**
          * An implementation of setReadWrite() that does nothing
          * whatsoever.
@@ -294,17 +291,11 @@ class PacketPane : public QWidget, public regina::NPacketListener {
         /**
          * Properties
          */
-        bool dirty;
         bool readWrite;
-        bool emergencyClosure;
-        bool emergencyRefresh;
-        bool isCommitting;
 
         /**
          * Internal actions
          */
-        QAction* actCommit;
-        QAction* actRefresh;
         QAction* actDockUndock;
         QAction* actClose;
 
@@ -347,19 +338,6 @@ class PacketPane : public QWidget, public regina::NPacketListener {
          */
         void fillPacketTypeMenu(QMenu* menu);
 
-        /**
-         * Does this packet pane contain any changes that have not yet
-         * been committed?
-         */
-        bool isDirty();
-
-        /**
-         * Signals that there are (or are not) changes in this interface
-         * that have not yet been committed.  External interface
-         * components will be updated accordingly.
-         */
-        void setDirty(bool newDirty);
-
         /**
          * Is this packet pane currently in read-write (as opposed to
          * read-only) mode?
@@ -426,72 +404,9 @@ class PacketPane : public QWidget, public regina::NPacketListener {
     public slots:
         /**
          * Queries the packet and refreshes the interface accordingly.
-         * Any uncommitted changes will be lost, though the user will be
-         * prompted first.
          */
         void refresh();
 
-        /**
-         * Like refresh(), except that the user is never prompted.
-         * Any uncommitted changes will be lost.
-         */
-        void refreshForce();
-
-        /**
-         * Commits any changes made in the user interface to the
-         * underlying packet.
-         *
-         * If the commit should not be allowed (i.e., the packet
-         * pane is in read-only mode or the underlying packet may not
-         * be modified for mathematical reasons), it will not take
-         * place.  Instead an appropriate error message will be displayed.
-         *
-         * Returns \c true if the commit took place (or if there were in
-         * fact no changes to commit), or \c false if the commit was not
-         * allowed to proceed.
-         */
-        bool commit();
-
-        /**
-         * Ensures that the underlying packet may be modified, and
-         * commits any changes made in the user interface.
-         *
-         * This routine behaves identically to commit(), with one
-         * exception.  Even if no changes have been made in the user
-         * interface, an error will still be raised if changes to the
-         * underlying packet are not allowed.
-         *
-         * This routine should be used when an operation plans to modify
-         * the underlying packet, but wishes to commit any changes made
-         * in the UI before proceeding.
-         *
-         * Returns \c true if changes are allowed to the underlying
-         * packet (and therefore any necessary commit took place), or
-         * \c false if changes are not allowed.
-         */
-        bool commitToModify();
-
-        /**
-         * Tries to commit any changes made in the user interface to the
-         * underlying packet, but allows the user to continue if the
-         * commit is not allowed.
-         *
-         * This routine behaves identically to commit(), except that if
-         * the commit is not allowed then the error messages will be
-         * replaced by gentle warnings explaining that an old copy of
-         * the packet is to be used.  The user will be offered the
-         * chance to continue or cancel.
-         *
-         * This routine should be used when an operation would like to
-         * commit current changes made in the UI, but if the commit fails
-         * then it can happily work with the last committed copy.
-         *
-         * Returns \c false if the user chose to cancel the operation,
-         * or \c true if either the commit succeeded or the user was
-         * happy to continue regardless.
-         */
-        bool tryCommit();
-
         /**
          * Closes this packet pane.  The user will be prompted if
          * necessary.
@@ -504,12 +419,6 @@ class PacketPane : public QWidget, public regina::NPacketListener {
          */
         bool close();
 
-        /**
-         * Closes this packet pane without prompting the user and
-         * without the chance of the closure being cancelled.
-         */
-        void closeForce();
-
         /**
          * Docks this packet pane into the main window, if
          * it is not already docked.  If another packet pane is
@@ -574,6 +483,9 @@ inline PacketUI::PacketUI(PacketPane* newEnclosingPane) :
 inline PacketUI::~PacketUI() {
 }
 
+inline void PacketUI::endEdit() {
+}
+
 inline PacketEditIface* PacketUI::getEditIface() {
     return 0;
 }
@@ -582,18 +494,10 @@ inline const QLinkedList<QAction*>& PacketUI::getPacketTypeActions() {
     return noActions;
 }
 
-inline void PacketUI::setDirty(bool newDirty) {
-    enclosingPane->setDirty(newDirty);
-}
-
 inline PacketReadOnlyUI::PacketReadOnlyUI(PacketPane* newEnclosingPane) :
         PacketUI(newEnclosingPane) {
 }
 
-inline void PacketReadOnlyUI::commit() {
-    setDirty(false);
-}
-
 inline void PacketReadOnlyUI::setReadWrite(bool) {
 }
 
@@ -609,12 +513,12 @@ inline PacketUI* PacketPane::getUI() {
     return mainUI;
 }
 
-inline bool PacketPane::isDirty() {
-    return dirty;
-}
-
 inline bool PacketPane::isReadWrite() {
     return readWrite;
 }
 
+inline void PacketPane::refresh() {
+    mainUI->refresh();
+}
+
 #endif
