@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -240,7 +240,6 @@ ReginaPrefSet::ReginaPrefSet() :
         pythonAutoIndent(true),
         pythonSpacesPerTab(4),
         pythonWordWrap(false),
-        snapPeaClosed(false),
         surfacesCompatThreshold(100),
         surfacesCreationCoords(regina::NS_STANDARD),
         surfacesCreationList(regina::NS_LIST_DEFAULT),
@@ -252,6 +251,8 @@ ReginaPrefSet::ReginaPrefSet() :
         tabDim3Tri(0),
         tabDim3TriAlgebra(0),
         tabDim3TriSkeleton(0),
+        tabSnapPeaTri(0),
+        tabSnapPeaTriAlgebra(0),
         tabSurfaceList(0),
         triGAPExec(defaultGAPExec),
         triGraphvizExec(defaultGraphvizExec),
@@ -449,24 +450,6 @@ void ReginaPrefSet::readInternal() {
     displayTagsInTree = settings.value("DisplayTagsInTree", false).toBool();
     settings.endGroup();
 
-    settings.beginGroup("Census");
-    censusFiles.clear();
-    // System census files:
-    QString exDir = QFile::decodeName(regina::NGlobalDirs::examples().c_str());
-    censusFiles.push_back(ReginaFilePref(exDir + "/closed-or-census.rga",
-        tr("Closed orientable census"), "ClosedOr", settings));
-    censusFiles.push_back(ReginaFilePref(exDir + "/closed-nor-census.rga",
-        tr("Closed non-orientable census"), "ClosedNor", settings));
-    censusFiles.push_back(ReginaFilePref(exDir + "/knot-link-census.rga",
-        tr("Hyperbolic knot/link census"), "KnotLink", settings));
-    censusFiles.push_back(ReginaFilePref(exDir + "/snappea-census.rga",
-        tr("Cusped hyperbolic census"), "CuspedHyp", settings));
-    censusFiles.push_back(ReginaFilePref(exDir + "/closed-hyp-census.rga",
-        tr("Closed hyperbolic census"), "ClosedHyp", settings));
-    // Additional user census files:
-    ReginaFilePref::readUserKey(censusFiles, settings);
-    settings.endGroup();
-
     settings.beginGroup("File");
     fileRecentMax = settings.value("RecentMax", 10).toInt();
     fileImportExportCodec = settings.value("ImportExportCodec",
@@ -484,7 +467,6 @@ void ReginaPrefSet::readInternal() {
     settings.endGroup();
 
     settings.beginGroup("SnapPea");
-    snapPeaClosed = settings.value("AllowClosed", false).toBool();
 #ifndef EXCLUDE_SNAPPEA
     regina::NSnapPeaTriangulation::enableKernelMessages(
         settings.value("KernelMessages", false).toBool());
@@ -518,6 +500,8 @@ void ReginaPrefSet::readInternal() {
     tabDim3Tri = settings.value("Dim3Tri", 0).toUInt();
     tabDim3TriAlgebra = settings.value("Dim3TriAlgebra", 0).toUInt();
     tabDim3TriSkeleton = settings.value("Dim3TriSkeleton", 0).toUInt();
+    tabSnapPeaTri = settings.value("SnapPeaTri", 0).toUInt();
+    tabSnapPeaTriAlgebra = settings.value("SnapPeaTriAlgebra", 0).toUInt();
     tabSurfaceList = settings.value("SurfaceList", 0).toUInt();
     settings.endGroup();
 
@@ -537,7 +521,7 @@ void ReginaPrefSet::readInternal() {
     readPythonLibraries();
 
     settings.beginGroup("Window");
-    windowMainSize = settings.value("MainSize", QSize()).toSize();
+    windowMainSize = settings.value("MainSizeV2", QSize()).toSize();
     windowPythonSize = settings.value("PythonSize", QSize()).toSize();
     settings.endGroup();
 
@@ -569,10 +553,6 @@ void ReginaPrefSet::saveInternal() const {
     settings.setValue("DisplayTagsInTree", displayTagsInTree);
     settings.endGroup();
 
-    settings.beginGroup("Census");
-    ReginaFilePref::writeKeys(censusFiles, settings);
-    settings.endGroup();
-
     settings.beginGroup("File");
     settings.setValue("RecentMax", fileRecentMax);
     settings.setValue("ImportExportCodec", fileImportExportCodec);
@@ -589,7 +569,6 @@ void ReginaPrefSet::saveInternal() const {
     settings.endGroup();
 
     settings.beginGroup("SnapPea");
-    settings.setValue("AllowClosed", snapPeaClosed);
 #ifndef EXCLUDE_SNAPPEA
     settings.setValue("KernelMessages",
         regina::NSnapPeaTriangulation::kernelMessagesEnabled());
@@ -619,6 +598,8 @@ void ReginaPrefSet::saveInternal() const {
     settings.setValue("Dim3Tri", tabDim3Tri);
     settings.setValue("Dim3TriAlgebra", tabDim3TriAlgebra);
     settings.setValue("Dim3TriSkeleton", tabDim3TriSkeleton);
+    settings.setValue("SnapPeaTri", tabSnapPeaTri);
+    settings.setValue("SnapPeaTriAlgebra", tabSnapPeaTriAlgebra);
     settings.setValue("SurfaceList", tabSurfaceList);
     settings.endGroup();
 
@@ -640,7 +621,7 @@ void ReginaPrefSet::saveInternal() const {
     savePythonLibraries();
 
     settings.beginGroup("Window");
-    settings.setValue("MainSize", windowMainSize);
+    settings.setValue("MainSizeV2", windowMainSize);
     settings.setValue("PythonSize", windowPythonSize);
     settings.endGroup();
 
