@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  KDE User Interface                                                    *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -351,8 +351,6 @@ class ReginaPrefSet : public QObject {
         bool anglesCreationTaut;
             /**< When enumerating angle structures, should the taut
                  structures option be enabled by default? */
-        QList<ReginaFilePref> censusFiles;
-            /**< The list of data files to use for census lookups. */
         bool displayTagsInTree;
             /**< Should we display packet tags in the visual tree? */
         QByteArray fileImportExportCodec;
@@ -375,9 +373,6 @@ class ReginaPrefSet : public QObject {
                  python console. */
         bool pythonWordWrap;
             /**< Should python consoles be word wrapped? */
-        bool snapPeaClosed;
-            /**< Do we allow the SnapPea kernel to work with closed (non-ideal)
-                 triangulations, even though it's dangerous? */
         unsigned surfacesCompatThreshold;
             /**< The maximum number of normal surfaces in a list for which the
                  compatibility matrices will be automatically calculated. */
@@ -406,6 +401,12 @@ class ReginaPrefSet : public QObject {
         unsigned tabDim3TriSkeleton;
             /**< The index of the initial sub-tab to open in a 3-manifold
                  triangulation skeleton viewer. */
+        unsigned tabSnapPeaTri;
+            /**< The index of the initial tab to open in a SnapPea
+                 triangulation viewer. */
+        unsigned tabSnapPeaTriAlgebra;
+            /**< The index of the initial sub-tab to open in a SnapPea
+                 triangulation algebra viewer. */
         unsigned tabSurfaceList;
             /**< The index of the initial sub-tab to open in a normal
                  surface list viewer. */
@@ -438,7 +439,11 @@ class ReginaPrefSet : public QObject {
                  that are not embedded */
 
         QSize windowMainSize;
-            /**< The initial size of a new main topology data window. */
+            /**< The initial size of a new main topology data window.
+                 Note: as of Regina 4.96, the key for this option in the
+                 configuration file has changed from \c MainSize to
+                 \c MainSizeV2 (since a new packet type was added, and
+                 so the window needs to be a little wider). */
         QSize windowPythonSize;
             /**< The initial size of a new python console. */
 
