@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Test Suite                                                            *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -46,14 +46,17 @@
  */
 
 namespace regina {
+    class Dim2Triangulation;
     class NTriangulation;
 }
 
+typedef void (*Dim2TriangulationTestFunction)(regina::Dim2Triangulation*);
 typedef void (*NTriangulationTestFunction)(regina::NTriangulation*);
 
 void runCensusMinClosed(NTriangulationTestFunction f, bool small_ = false);
 void runCensusAllClosed(NTriangulationTestFunction f, bool small_ = false);
 void runCensusAllBounded(NTriangulationTestFunction f, bool small_ = false);
 void runCensusAllIdeal(NTriangulationTestFunction f, bool small_ = false);
+void runCensusAllNoBdry(NTriangulationTestFunction f, bool small_ = false);
 
 #endif
