@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Test Suite                                                            *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -40,8 +40,10 @@
 #include <cppunit/TestResult.h>
 #include <cppunit/TextTestProgressListener.h>
 #include "testsuite/testparams.h"
+#include "testsuite/algebra/testalgebra.h"
 #include "testsuite/angle/testangle.h"
 #include "testsuite/census/testcensus.h"
+#include "testsuite/dim2/testdim2.h"
 #include "testsuite/maths/testmaths.h"
 #ifndef EXCLUDE_SNAPPEA
 #include "testsuite/snappea/testsnappea.h"
@@ -138,6 +140,12 @@ bool runAllTests() {
     addMatrixOps(runner);
     addPermConv(runner);
 
+    // Algebra:
+    addNGroupPresentation(runner);
+
+    // Dim2Triangulation:
+    addDim2Triangulation(runner);
+
     // Triangulation:
     addNTriangulation(runner);
     addElementaryMoves(runner);
