@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Test Suite                                                            *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -35,9 +35,13 @@
 #include <cmath>
 #include <iomanip>
 #include <cppunit/extensions/HelperMacros.h>
+#include "maths/nmatrixint.h"
 #include "snappea/nsnappeatriangulation.h"
+#include "surfaces/nnormalsurfacelist.h"
 #include "triangulation/nexampletriangulation.h"
 #include "triangulation/ntriangulation.h"
+
+#include "testsuite/exhaustive.h"
 #include "testsuite/snappea/testsnappea.h"
 
 using regina::NExampleTriangulation;
@@ -53,6 +57,8 @@ class NSnapPeaTriangulationTest : public CppUnit::TestFixture {
     CPPUNIT_TEST(volume);
     CPPUNIT_TEST(flat);
     CPPUNIT_TEST(degenerate);
+    CPPUNIT_TEST(spunBoundaries);
+    CPPUNIT_TEST(stability);
 
     CPPUNIT_TEST_SUITE_END();
 
@@ -145,10 +151,17 @@ class NSnapPeaTriangulationTest : public CppUnit::TestFixture {
             n4_9_2.insertRehydration("ebdbcdddcemre");
             n4_1_2_1.insertRehydration("eahbcdddjxxxj");
 
+            // Note: the non-orientable manifold below is the same as
+            // NExampleTriangulation::smallClosedNonOrblHyperbolic()),
+            // but if we build it from NExampleTriangulation then we
+            // seem to get a degenerate solution.  Using the isosig
+            // gives the same triangulation with a different labelling,
+            // which seems to prod SnapPea into finding a better
+            // (non_geometric) solution instead.
             copyAndDelete(closedHypOr,
                 NExampleTriangulation::smallClosedOrblHyperbolic());
-            copyAndDelete(closedHypNor,
-                NExampleTriangulation::smallClosedNonOrblHyperbolic());
+            copyAndDelete(closedHypNor, NTriangulation::fromIsoSig(
+                "lLLLALAQccegffiijkikkkknawmhvwcls"));
 
             copyAndDelete(weberSeifert,
                 NExampleTriangulation::weberSeifert());
@@ -298,59 +311,6 @@ class NSnapPeaTriangulationTest : public CppUnit::TestFixture {
             testIncompatible(edgeInvalid,
                 "A triangulation with two invalid edges "
                 "should not be representable in SnapPea format.");
-
-            CPPUNIT_ASSERT_MESSAGE(
-                "The small closed orientable hyperbolic triangulation "
-                "appears to have been incorrectly constructed.",
-                closedHypOr.isValid() &&
-                closedHypOr.isConnected() &&
-                closedHypOr.isOrientable() &&
-                (! closedHypOr.isIdeal()) &&
-                closedHypOr.isStandard() &&
-                (! closedHypOr.hasBoundaryTriangles()));
-            testIncompatible(closedHypOr,
-                "A closed orientable hyperbolic triangulation "
-                "should not be representable in SnapPea format.");
-
-            CPPUNIT_ASSERT_MESSAGE(
-                "The small closed non-orientable hyperbolic triangulation "
-                "appears to have been incorrectly constructed.",
-                closedHypNor.isValid() &&
-                closedHypNor.isConnected() &&
-                (! closedHypNor.isOrientable()) &&
-                (! closedHypNor.isIdeal()) &&
-                closedHypNor.isStandard() &&
-                (! closedHypNor.hasBoundaryTriangles()));
-            testIncompatible(closedHypNor,
-                "A closed non-orientable hyperbolic triangulation "
-                "should not be representable in SnapPea format.");
-
-            CPPUNIT_ASSERT_MESSAGE(
-                "The Weber-Seifert dodecahedral space "
-                "appears to have been incorrectly constructed.",
-                weberSeifert.isValid() &&
-                weberSeifert.isConnected() &&
-                weberSeifert.isOrientable() &&
-                (! weberSeifert.isIdeal()) &&
-                weberSeifert.isStandard() &&
-                (! weberSeifert.hasBoundaryTriangles()));
-            testIncompatible(weberSeifert,
-                "The Weber-Seifert dodecahedral space is closed, and so "
-                "should not be representable in SnapPea format.");
-
-            CPPUNIT_ASSERT_MESSAGE(
-                "The cusped solid torus with finite vertex "
-                "appears to have been incorrectly constructed.",
-                cuspedTorus.isValid() &&
-                cuspedTorus.isConnected() &&
-                cuspedTorus.isOrientable() &&
-                cuspedTorus.isIdeal() &&
-                cuspedTorus.isStandard() &&
-                (! cuspedTorus.hasBoundaryTriangles()) &&
-                cuspedTorus.getNumberOfTetrahedra() == 3);
-            testIncompatible(cuspedTorus,
-                "A cusped solid torus with an additional finite vertex "
-                "should not be representable in SnapPea format.");
         }
 
         void testVolume(NTriangulation& tri, const char* triName,
@@ -410,9 +370,9 @@ class NSnapPeaTriangulationTest : public CppUnit::TestFixture {
             testVolume(n4_9_2,   "N 4_9^2",   4.0597664256, 9);
             testVolume(n4_1_2_1, "N 4_1^2,1", 3.6638623767, 9);
 
-            // testVolume(closedHypOr, "or_0.94270736", 0.94270736, 7);
-            // testVolume(closedHypNor, "nor_2.02988321", 2.02988121, 7);
-            // testVolume(weberSeifert, "Weber-Seifert", 11.1990647, 6);
+            testVolume(closedHypOr, "or_0.94270736", 0.94270736, 7);
+            testVolume(closedHypNor, "nor_2.02988321", 2.02988321, 7);
+            testVolume(weberSeifert, "Weber-Seifert", 11.1990647, 6);
         }
 
         void testZeroVolume(const char* triName, double foundVol,
@@ -494,8 +454,7 @@ class NSnapPeaTriangulationTest : public CppUnit::TestFixture {
             testFlat(flatNor, "The non-orientable flat triangulation", 9);
         }
 
-        void testDegenerate(NTriangulation& tri, const char* triName,
-                int maxPlaces) {
+        void testDegenerate(NTriangulation& tri, const char* triName) {
             // Verify that the triangulation has a degenerate solution
             // and the volume is zero.  The volume is tested to whatever
             // precision is reported (up to a maximum of maxPlaces),
@@ -522,21 +481,88 @@ class NSnapPeaTriangulationTest : public CppUnit::TestFixture {
                     NSnapPeaTriangulation::degenerate_solution);
             }
 
-            int precision;
-            double foundVol = s.volume(precision);
-
-            // Dumb down the precision to our given maximum.
-            if (precision > maxPlaces)
-                precision = maxPlaces;
-
-            testZeroVolume(triName, foundVol, precision);
+            // Don't test volumes for degenerate solutions, since these
+            // can go all over the shop.
         }
 
         void degenerate() {
             testDegenerate(degenerateOr,
-                "The orientable degenerate triangulation", 9);
+                "The orientable degenerate triangulation");
             testDegenerate(degenerateNor,
-                "The non-orientable degenerate triangulation", 9);
+                "The non-orientable degenerate triangulation");
+            testDegenerate(cuspedTorus,
+                "A cusped solid torus with an additional finite vertex");
+        }
+
+        void spunBoundaries() {
+            NTriangulation* f8 =
+                NExampleTriangulation::figureEightKnotComplement();
+            NSnapPeaTriangulation* t = new NSnapPeaTriangulation(*f8);
+
+            regina::NNormalSurfaceList* s =
+                regina::NNormalSurfaceList::enumerate(t, regina::NS_QUAD);
+            if (s->getNumberOfSurfaces() != 4)
+                CPPUNIT_FAIL(
+                    "The figure 8 knot complement should have 4 vertex "
+                    "surfaces in quad space.");
+
+            regina::NMatrixInt* m;
+            bool found[4];
+            for (int i = 0; i < s->getNumberOfSurfaces(); ++i) {
+                m = s->getSurface(i)->boundaryIntersections();
+                if (m->rows() != 1 || m->columns() != 2) {
+                    CPPUNIT_FAIL(
+                        "Figure 8 knot complement: boundaryIntersections() "
+                        "should give 1x2 matrices.");
+                }
+                if (m->entry(0, 0) == 1 && m->entry(0, 1) == 4)
+                    found[0] = true;
+                else if (m->entry(0, 0) == 1 && m->entry(0, 1) == -4)
+                    found[1] = true;
+                else if (m->entry(0, 0) == -1 && m->entry(0, 1) == 4)
+                    found[2] = true;
+                else if (m->entry(0, 0) == -1 && m->entry(0, 1) == -4)
+                    found[3] = true;
+                else {
+                    std::ostringstream msg;
+                    msg << "Figure 8 knot complement: boundaryIntersections() "
+                        "gives unexpected result ("
+                        << m->entry(0, 0) << ", " << m->entry(0, 1) << ").";
+                    CPPUNIT_FAIL(msg.str());
+                }
+                delete m;
+            }
+
+            if (! found[0])
+                CPPUNIT_FAIL("Figure 8 knot complement: did not find "
+                    "boundary slope (1, 4).");
+            if (! found[1])
+                CPPUNIT_FAIL("Figure 8 knot complement: did not find "
+                    "boundary slope (1, -4).");
+            if (! found[2])
+                CPPUNIT_FAIL("Figure 8 knot complement: did not find "
+                    "boundary slope (-1, 4).");
+            if (! found[3])
+                CPPUNIT_FAIL("Figure 8 knot complement: did not find "
+                    "boundary slope (-1, -4).");
+
+            delete s;
+            delete t;
+            delete f8;
+        }
+
+        static void testStability(NTriangulation* tri) {
+            // Just make sure SnapPea can work with the triangulation
+            // without crashing.
+            NSnapPeaTriangulation s(*tri);
+            s.volume();
+            s.randomize();
+            s.volume();
+            NTriangulation t(s);
+        }
+
+        void stability() {
+            runCensusAllNoBdry(&testStability);
         }
 };
 
