@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Form a census of triangulations that satisfy given properties         *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -40,15 +40,12 @@
 #include <sstream>
 #include <popt.h>
 #include <unistd.h>
-#include "census/dim2census.h"
 #include "census/dim2edgepairing.h"
 #include "census/dim2gluingpermsearcher.h"
 #if SUPPORT_DIM4
-#include "census/dim4census.h"
 #include "census/dim4facetpairing.h"
 #include "census/dim4gluingpermsearcher.h"
 #endif
-#include "census/ncensus.h"
 #include "census/nfacepairing.h"
 #include "census/ngluingpermsearcher.h"
 #include "dim2/dim2triangulation.h"
@@ -80,6 +77,7 @@ regina::NBoolSet
 int minimal = 0;
 int minimalPrime = 0;
 int minimalPrimeP2 = 0;
+int minimalHyp = 0;
 int dim2 = 0;
 int dim4 = 0;
 int usePairs = 0;
@@ -141,7 +139,7 @@ struct Dim3Params {
     }
 
     inline static bool mightBeMinimal(Triangulation* tri) {
-        return regina::NCensus::mightBeMinimal(tri, 0);
+        return ! tri->simplifyToLocalMinimum(false);
     }
 
     inline static const Pairing* pairingFor(const GluingPermSearcher* s) {
@@ -182,6 +180,13 @@ void foundGluingPerms(const typename CensusType::GluingPermSearcher* perms,
     if (perms) {
         typename CensusType::Triangulation* tri = perms->triangulate();
 
+        // For minimalHyp, we don't run CensusType::mightBeMinimal().
+        // This is because mightBeMinimal() only tests for immediate
+        // reductions (i.e., it doesn't use 4-4 moves or well-climbing
+        // techniques), and NHyperbolicMinSearcher already ensures that
+        // no such moves are possible (since it ensures no internal vertices
+        // and no low-degree edges).
+
         bool ok = true;
         if (! tri->isValid())
             ok = false;
@@ -305,12 +310,16 @@ regina::NText* parameterPacket() {
     else
         descStream << "Orientable and non-orientable\n";
 
-    if (minimalPrimeP2)
+    if (minimalHyp)
+        descStream << "Ignored triangulations that are obviously not "
+            "minimal ideal triangulations of cusped finite-volume "
+            "hyperbolic 3-manifolds\n";
+    else if (minimalPrimeP2)
         descStream << "Ignored obviously non-minimal, non-prime, "
-            << "disc-reducible and/or P2-reducible triangulations\n";
+            "disc-reducible and/or P2-reducible triangulations\n";
     else if (minimalPrime)
         descStream << "Ignored obviously non-minimal, non-prime and/or "
-            << "disc-reducible triangulations\n";
+            "disc-reducible triangulations\n";
     else if (minimal)
         descStream << "Ignored obviously non-minimal triangulations\n";
 
@@ -353,6 +362,10 @@ int main(int argc, const char* argv[]) {
             "Ignore obviously non-minimal, non-prime and/or disc-reducible triangulations.", 0 },
         { "minprimep2", 'N', POPT_ARG_NONE, &minimalPrimeP2, 0,
             "Ignore obviously non-minimal, non-prime, disc-reducible and/or P2-reducible triangulations.", 0 },
+        { "minhyp", 'h', POPT_ARG_NONE, &minimalHyp, 0,
+            "Ignore triangulations that are obviously not minimal ideal "
+            "triangulations of cusped finite-volume hyperbolic 3-manifolds.  "
+            "Implies --internal and --ideal.", 0 },
         { "dim2", '2', POPT_ARG_NONE, &dim2, 0,
             "Run a census of 2-manifold triangulations, "
             "not 3-manifold triangulations.  Here --tetrahedra counts "
@@ -401,11 +414,22 @@ int main(int argc, const char* argv[]) {
         }
     }
 
+    // Some options imply others.
+    if (minimalHyp) {
+        argIdeal = 1;
+        if (! usePairs)
+            argNoBdry = 1;
+    }
+
     // Run a sanity check on the command-line arguments.
     bool broken = false;
     if ((! genPairs) && outFile.empty()) {
         std::cerr << "An output file must be specified.\n";
         broken = true;
+    } else if (genPairs &&
+            (minimal || minimalPrime || minimalPrimeP2 || minimalHyp)) {
+        std::cerr << "Minimality options cannot be used with -p/--genpairs.\n";
+        broken = true;
     } else if (genPairs && (argOr || argNor)) {
         std::cerr << "Orientability options cannot be used with "
             << "-p/--genpairs.\n";
@@ -413,23 +437,24 @@ int main(int argc, const char* argv[]) {
     } else if (genPairs && (argFinite || argIdeal)) {
         std::cerr << "Finiteness options cannot be used with -p/--genpairs.\n";
         broken = true;
-    } else if (genPairs && (minimal || minimalPrime || minimalPrimeP2)) {
-        std::cerr << "Minimality options cannot be used with -p/--genpairs.\n";
-        broken = true;
     } else if (genPairs && sigs) {
         std::cerr << "Signature output cannot be used with -p/--genpairs.\n";
         broken = true;
     } else if (dim2 && dim4) {
         std::cerr << "Options -2/--dim2 and -4/--dim4 cannot be used together.\n";
         broken = true;
-    } else if (dim2 && (argFinite || argIdeal)) {
-        std::cerr << "Finiteness options cannot be used with -2/--dim2.\n";
+    } else if (dim2 && minimalHyp) {
+        std::cerr << "Hyperbolicity options cannot be used with -2/--dim2.\n";
         broken = true;
     } else if (dim2 && (minimalPrime || minimalPrimeP2)) {
         std::cerr << "Primeness options cannot be used with -2/--dim2 "
             "(the weaker -m/--minimal can).\n";
         broken = true;
-    } else if (dim4 && (minimal || minimalPrime || minimalPrimeP2)) {
+    } else if (dim2 && (argFinite || argIdeal)) {
+        std::cerr << "Finiteness options cannot be used with -2/--dim2.\n";
+        broken = true;
+    } else if (dim4 &&
+            (minimal || minimalPrime || minimalPrimeP2 || minimalHyp)) {
         std::cerr << "Minimality options cannot be used with -4/--dim4.\n";
         broken = true;
     } else if (usePairs && nTet) {
@@ -447,6 +472,10 @@ int main(int argc, const char* argv[]) {
         std::cerr << "The number of " << WORD_tetrahedra
             << " must be between 1 and " << MAXTET << " inclusive.\n";
         broken = true;
+    } else if (argBdry && minimalHyp) {
+        std::cerr << "Options -b/--boundary and -h/--minhyp "
+            << "cannot be used together.\n";
+        broken = true;
     } else if (argBdry && argNoBdry) {
         std::cerr << "Options -b/--boundary and -i/--internal "
             << "cannot be used together.\n";
@@ -455,6 +484,10 @@ int main(int argc, const char* argv[]) {
         std::cerr << "Options -o/--orientable and -n/--nonorientable "
             << "cannot be used together.\n";
         broken = true;
+    } else if (argFinite && minimalHyp) {
+        std::cerr << "Options -f/--finite and -h/--minhyp"
+            << "cannot be used together.\n";
+        broken = true;
     } else if (argFinite && argIdeal) {
         std::cerr << "Options -f/--finite and -d/--ideal "
             << "cannot be used together.\n";
@@ -510,7 +543,11 @@ int main(int argc, const char* argv[]) {
             broken = true;
         } else {
             // Asking for a valid positive number of boundary faces.
-            if (argNoBdry) {
+            if (minimalHyp) {
+                std::cerr << "Option -h/--minhyp cannot be used with "
+                    << "-B/--bdryfaces=<non-zero>.\n";
+                broken = true;
+            } else if (argNoBdry) {
                 std::cerr << "Option -i/--internal cannot be used with "
                     << "-B/--bdryfaces=<non-zero>.\n";
                 broken = true;
@@ -599,12 +636,14 @@ int runCensus() {
 
     // Start the census running.
     if (minimalPrimeP2)
-        whichPurge = regina::NCensus::PURGE_NON_MINIMAL_PRIME |
-            regina::NCensus::PURGE_P2_REDUCIBLE;
+        whichPurge = regina::NGluingPermSearcher::PURGE_NON_MINIMAL_PRIME |
+            regina::NGluingPermSearcher::PURGE_P2_REDUCIBLE;
     else if (minimalPrime)
-        whichPurge = regina::NCensus::PURGE_NON_MINIMAL_PRIME;
+        whichPurge = regina::NGluingPermSearcher::PURGE_NON_MINIMAL_PRIME;
+    else if (minimalHyp)
+        whichPurge = regina::NGluingPermSearcher::PURGE_NON_MINIMAL_HYP;
     else if (minimal)
-        whichPurge = regina::NCensus::PURGE_NON_MINIMAL;
+        whichPurge = regina::NGluingPermSearcher::PURGE_NON_MINIMAL;
 
     if (usePairs) {
         // Only use the face pairings read from standard input.
@@ -667,7 +706,7 @@ int runCensus() {
     if (sigs) {
         sigStream.close();
     } else {
-        if (! regina::writeXMLFile(outFile.c_str(), parent)) {
+        if (! parent->save(outFile.c_str())) {
             std::cerr << "Output file " << outFile
                 << " could not be written.\n";
             return 1;
