@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Compare two data files in search of isomorphic triangulations         *
  *                                                                        *
- *  Copyright (c) 1999-2013, Ben Burton                                   *
+ *  Copyright (c) 1999-2014, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -178,7 +178,7 @@ int main(int argc, char* argv[]) {
         usage(argv[0], "Two data files must be specified.");
 
     // Open the two data files.
-    NPacket* tree1 = regina::readFileMagic(file1);
+    NPacket* tree1 = regina::open(file1.c_str());
     if (! tree1) {
         std::cerr << "File " << file1 << " could not be read.\n";
         std::cerr << "Please check that it exists and that it is a Regina data file.\n";
@@ -186,7 +186,7 @@ int main(int argc, char* argv[]) {
         return 1;
     }
 
-    NPacket* tree2 = regina::readFileMagic(file2);
+    NPacket* tree2 = regina::open(file2.c_str());
     if (! tree2) {
         std::cerr << "File " << file2 << " could not be read.\n";
         std::cerr << "Please check that it exists and that it is a Regina data file.\n";
